! =================================================================================================================================
! MODULE       : stomate_turnover.f90
!
! CONTACT      : orchidee-help _at_ ipsl.jussieu.fr
!
! LICENCE      : IPSL (2006)
! This software is governed by the CeCILL licence see ORCHIDEE/ORCHIDEE_CeCILL.LIC
!
!>\BRIEF        This module manages the end of the growing season and calculates herbivory 
!               and turnover of leaves, fruits, fine roots.
!! 
!!\n DESCRIPTION: This subroutine calculates leaf senescence due to climatic conditions or 
!! as a function of leaf age and new LAI, and subsequent turnover of the different plant 
!! biomass compartments (sections 1 to 6), herbivory (section 7), fruit turnover for trees 
!! (section 8) and sapwood conversion (section 9). 
!!
!! RECENT CHANGE(S): None
!!
!! SVN          :
!! $HeadURL$
!! $Date$
!! $Revision$
!! \n
!_ ================================================================================================================================

MODULE stomate_turnover

  ! modules used:gg
  USE xios_orchidee
  USE ioipsl_para
  USE stomate_data
  USE constantes
  USE pft_parameters
  USE sapiens_agriculture, ONLY: crop_harvest
  USE function_library, ONLY : biomass_to_lai

  IMPLICIT NONE

  ! private & public routines

  PRIVATE
  PUBLIC turnover_diagnostic, turnover_prognostic, turnover_clear

  LOGICAL, SAVE                          :: firstcall = .TRUE.           !! first call (true/false)
!$OMP THREADPRIVATE(firstcall)

CONTAINS


!! ================================================================================================================================
!! SUBROUTINE   : turnover_clear
!!
!>\BRIEF        Set flag ::firstcall to .TRUE., and therefore activate section 1 
!!              of subroutine turn which writes a message to the output.
!!                
!_ ================================================================================================================================

  SUBROUTINE turnover_clear
    firstcall=.TRUE.
  END SUBROUTINE turnover_clear


!! ================================================================================================================================
!! SUBROUTINE    : turn
!!
!>\BRIEF         Calculate turnover of leaves, roots, fruits and sapwood due to aging or climatic 
!!               induced senescence. Calculate herbivory.
!!
!! DESCRIPTION : This subroutine determines the turnover of leaves and fine roots (and stems for grasses)
!! and simulates following processes:
!! 1. Mean leaf age is calculated from leaf ages of separate leaf age classes. Should actually 
!!    be recalculated at the end of the routine, but it does not change too fast. The mean leaf 
!!    age is calculated using the following equation:
!!    \latexonly
!!    \input{turnover_lma_update_eqn1.tex}
!!    \endlatexonly
!!    \n 
!! 2. Meteorological senescence: the detection of the end of the growing season and shedding 
!!    of leaves, fruits and fine roots due to unfavourable meteorological conditions.
!!    The model distinguishes three different types of "climatic" leaf senescence, that do not 
!!    change the age structure: sensitivity to cold temperatures, to lack of water, or both. 
!!    If meteorological conditions are fulfilled, a flag ::senescence is set to TRUE. Note
!!    that evergreen species do not experience climatic senescence.
!!    Climatic senescence is triggered by sensitivity to cold temperatures where the critical 
!!    temperature for senescence is calculated using the following equation:
!!    \latexonly
!!    \input{turnover_temp_crit_eqn2.tex}
!!    \endlatexonly
!!    \n
!!    Climatic senescence is triggered by sensitivity to lack of water availability where the 
!!    moisture availability critical level is calculated using the following equation:
!!    \latexonly
!!    \input{turnover_moist_crit_eqn3.tex}
!!    \endlatexonly
!!    \n
!!    Climatic senescence is triggered by sensitivity to temperature or to lack of water where
!!    critical temperature and moisture availability are calculated as above.\n
!!    Trees in climatic senescence lose their fine roots at the same rate as they lose their leaves. 
!!    The rate of biomass loss of both fine roots and leaves is presribed through the equation:
!!    \latexonly
!!    \input{turnover_clim_senes_biomass_eqn4.tex}
!!    \endlatexonly
!!    \n
!!    with ::leaffall(j) a PFT-dependent time constant which is given in 
!!    ::stomate_constants. In grasses, leaf senescence is extended to the whole plant 
!!    (all carbon pools) except to its carbohydrate reserve.    
!! 3. Senescence due to aging: the loss of leaves, fruits and  biomass due to aging
!!    At a certain age, leaves fall off, even if the climate would allow a green plant
!!    all year round. Even if the meteorological conditions are favorable for leaf maintenance,
!!    plants, and in particular, evergreen trees, have to renew their leaves simply because the 
!!    old leaves become inefficient. Roots, fruits (and stems for grasses) follow leaves. 
!!    The ??senescence?? rate varies with leaf age. Note that plant is not declared senescent 
!!    in this case (wchich is important for allocation: if the plant loses leaves because of 
!!    their age, it can renew them). The leaf turnover rate due to aging of leaves is calculated
!!    using the following equation:
!!    \latexonly
!!    \input{turnover_age_senes_biomass_eqn5.tex}
!!    \endlatexonly
!!    \n
!!    Drop all leaves if there is a very low leaf mass during senescence. After this, the biomass 
!!    of different carbon pools both for trees and grasses is set to zero and the mean leaf age 
!!    is reset to zero. Finally, the leaf fraction and leaf age of the different leaf age classes 
!!    is set to zero. For deciduous trees: next to leaves, also fruits and fine roots are dropped.
!!    For grasses: all aboveground carbon pools, except the carbohydrate reserves are affected:
!! 4. Update the leaf biomass, leaf age class fraction and the LAI
!!    Older leaves will fall more frequently than younger leaves and therefore the leaf age 
!!    distribution needs to be recalculated after turnover. The fraction of biomass in each 
!!    leaf class is updated using the following equation:
!!    \latexonly
!!    \input{turnover_update_LeafAgeDistribution_eqn6.tex}
!!    \endlatexonly
!!    \n
!! 5. Simulate herbivory activity and update leaf and fruits biomass. Herbivore activity 
!!    affects the biomass of leaves and fruits as well as stalks (only for grasses).
!!    However, herbivores do not modify leaf age structure.
!! 6. Calculates fruit turnover for trees. Trees simply lose their fruits with a time 
!!    constant ::tau_fruit(j), that is set to 90 days for all PFTs in ::stomate_constants 
!! 7. Convert sapwood to heartwood for trees and update heart and softwood above and 
!!    belowground biomass. Sapwood biomass is converted into heartwood biomass 
!!    with a time constant tau ::tau_sap(j) of 1 year. Note that this biomass conversion 
!!    is not added to "turnover" as the biomass is not lost. For the updated heartwood, 
!!    the sum of new heartwood above and new heartwood below after converting sapwood to 
!!    heartwood, is saved as ::hw_new(:). Creation of new heartwood decreases the age of 
!!    the plant ??carbon?? with a factor that is determined by: old heartwood ::hw_old(:) 
!!    divided by the new heartwood ::hw_new(:)
!!
!! RECENT CHANGE(S) : None
!!
!! MAIN OUTPUT VARIABLES: ::Biomass of leaves, fruits, fine roots and sapwood above (latter for grasses only), 
!!                        ::Update LAI, ::Update leaf age distribution with new leaf age class fraction 
!!
!! REFERENCE(S) : 
!! - Krinner, G., N. Viovy, N. de Noblet-Ducoudre, J. Ogee, J. Polcher, P. 
!! Friedlingstein, P. Ciais, S. Sitch and I.C. Prentice (2005), A dynamic global
!! vegetation model for studies of the coupled atmosphere-biosphere system, Global
!! Biogeochemical Cycles, 19, doi:10.1029/2003GB002199. 
!! - McNaughton, S. J., M. Oesterheld, D. A. Frank and K. J. Williams (1989), 
!! Ecosystem-level patterns of primary productivity and herbivory in terrestrial habitats, 
!! Nature, 341, 142-144, 1989. 
!! - Sitch, S., C. Huntingford, N. Gedney, P. E. Levy, M. Lomas, S. L. Piao, , Betts, R., Ciais, P., Cox, P., 
!! Friedlingstein, P., Jones, C. D., Prentice, I. C. and F. I. Woodward : Evaluation of the terrestrial carbon  
!! cycle, future plant geography and climate-carbon cycle feedbacks using 5 dynamic global vegetation 
!! models (dgvms), Global Change Biology, 14(9), 2015–2039, 2008. 
!!
!! FLOWCHART    : 
!! \latexonly
!! \includegraphics[scale=0.5]{turnover_flowchart_1.png}
!! \includegraphics[scale=0.5]{turnover_flowchart_2.png}
!! \endlatexonly
!! \n
!_ ================================================================================================================================

  SUBROUTINE turnover_diagnostic (npts, dt, PFTpresent, herbivores, maxmoiavail_lastyear, &
       minmoiavail_lastyear, moiavail_week, moiavail_month, tlong_ref, t2m_month, &
       t2m_week, veget_max, gdd_from_growthinit, leaf_age, leaf_frac, & 
       age, lai, biomass, turnover, senescence, &
       turnover_time, forest_managed)

    !! 0. Variable and parameter declaration

    !! 0.1 Input variables 

    INTEGER(i_std), INTENT(in)                                 :: npts                 !! Domain size - number of grid cells 
                                                                                       !! (unitless)
    INTEGER(i_std), DIMENSION (npts,nvm), INTENT(in)           :: forest_managed       !! forest management flag 
    REAL(r_std), INTENT(in)                                    :: dt                   !! time step (dt_days)
    LOGICAL, DIMENSION(npts,nvm), INTENT(in)                   :: PFTpresent           !! PFT exists (true/false)
    REAL(r_std), DIMENSION(npts), INTENT(in)                   :: tlong_ref            !! "long term" 2 meter reference 
                                                                                       !! temperatures (K) 
    REAL(r_std), DIMENSION(npts), INTENT(in)                   :: t2m_month            !! "monthly" 2-meter temperatures (K)
    REAL(r_std), DIMENSION(npts), INTENT(in)                   :: t2m_week             !! "weekly" 2 meter temperatures (K)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)               :: herbivores           !! time constant of probability of a leaf to 
                                                                                       !! be eaten by a herbivore (days) 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)               :: maxmoiavail_lastyear !! last year's maximum moisture availability 
                                                                                       !! (0-1, unitless)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)               :: minmoiavail_lastyear !! last year's minimum moisture availability 
                                                                                       !! (0-1, unitless)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)               :: moiavail_week        !! "weekly" moisture availability 
                                                                                       !! (0-1, unitless)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)               :: moiavail_month       !! "monthly" moisture availability 
                                                                                       !! (0-1, unitless)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)               :: veget_max            !! "maximal" coverage fraction of a PFT (LAI 
                                                                                       !! -> infinity) on ground (unitless) 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)               :: gdd_from_growthinit  !! gdd senescence for crop

    !! 0.2 Output variables

    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements), INTENT(out) :: turnover         !! Turnover @tex ($gC m^{-2}$) @endtex
    LOGICAL, DIMENSION(npts,nvm), INTENT(out)                  :: senescence           !! is the plant senescent? (true/false) 
                                                                                       !! (interesting only for deciduous trees: 
                                                                                       !! carbohydrate reserve) 
    !! 0.3 Modified variables

    REAL(r_std), DIMENSION(npts,nvm,nleafages), INTENT(inout)  :: leaf_age             !! age of the leaves (days)
    REAL(r_std), DIMENSION(npts,nvm,nleafages), INTENT(inout)  :: leaf_frac            !! fraction of leaves in leaf age class 
                                                                                       !! (0-1, unitless)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)            :: age                  !! age (years)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)               :: lai                  !! leaf area index @tex ($m^2 m^{-2}$) 
                                                                                       !! @endtex
    REAL(r_std), DIMENSION(npts,nvm,nparts), INTENT(inout)     :: turnover_time        !! turnover_time of grasses (days) 
    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements), &
                                                 INTENT(inout) :: biomass              !! biomass @tex ($gC m^{-2}$) @endtex

    !! 0.4 Local  variables

    REAL(r_std), DIMENSION(npts,nvm)                           :: leaf_meanage         !! mean age of the leaves (days)
    REAL(r_std), DIMENSION(npts)                               :: dturnover            !! Intermediate variable for turnover ??
                                                                                       !! @tex ($gC m^{-2}$) @endtex 
    REAL(r_std), DIMENSION(npts)                               :: moiavail_crit        !! critical moisture availability, function 
                                                                                       !! of last year's moisture availability 
                                                                                       !! (0-1, unitless)
    REAL(r_std), DIMENSION(npts)                               :: tl                   !! long term annual mean temperature, (C)
    REAL(r_std), DIMENSION(npts)                               :: t_crit               !! critical senescence temperature, function 
                                                                                       !! of long term annual temperature (K) 
    LOGICAL, DIMENSION(npts)                                   :: shed_rest            !! shed the remaining leaves? (true/false)
    REAL(r_std), DIMENSION(npts)                               :: sapconv              !! Sapwood conversion @tex ($gC m^{-2}$) 
                                                                                       !! @endtex 
    REAL(r_std), DIMENSION(npts)                               :: hw_old               !! old heartwood mass @tex ($gC m^{-2}$) 
                                                                                       !! @endtex 
    REAL(r_std), DIMENSION(npts)                               :: hw_new               !! new heartwood mass @tex ($gC m^{-2}$) 
                                                                                       !! @endtex 
    REAL(r_std), DIMENSION(npts)                               :: lm_old               !! old leaf mass @tex ($gC m^{-2}$) @endtex
    REAL(r_std), DIMENSION(npts,nleafages)                     :: delta_lm             !! leaf mass change for each age class @tex 
                                                                                       !! ($gC m^{-2}$) @endtex 
    REAL(r_std), DIMENSION(npts)                               :: turnover_rate        !! turnover rate (unitless) 
    REAL(r_std), DIMENSION(npts,nvm)                           :: leaf_age_crit        !! critical leaf age (days)
    REAL(r_std), DIMENSION(npts,nvm)                           :: new_turnover_time    !! instantaneous turnover time (days)
    INTEGER(i_std)                                             :: j,m,k                !! Index (unitless)
    REAL(r_std)                                                :: branch_turn          !! turnover of branches (how much goes 
                                                                                       !! to litter each day) (cf article CO2fix)
    REAL(r_std), SAVE                                          :: ss_branch_turn       !! Variations for sensitivity analysis
!$OMP THREADPRIVATE(ss_branch_turn)
    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements)          :: bm_old               !! old root mass
    REAL(r_std), DIMENSION(npts,nvm)                           :: sapwood_age          !! sapwood age (days)
    REAL(r_std), DIMENSION(npts)                               :: sw_old               !! old sapwood mass 
                                                                                       !! (gC/(m**2 of nat/agri ground))
    REAL(r_std), DIMENSION(npts)                               :: sw_new               !! new sapwood mass 
                                                                                       !! (gC/(m**2 of nat/agri ground))
    INTEGER(i_std), SAVE                                       :: turn_count           !!  
!$OMP THREADPRIVATE(turn_count)
!_ ================================================================================================================================

    IF (bavard.GE.3) WRITE(numout,*) 'Entering turnover'

    !! 1. first call - output messages

    IF ( firstcall ) THEN

       WRITE(numout,*) 'turnover:'

       WRITE(numout,*) ' > minimum mean leaf age for senescence (days) (::min_leaf_age_for_senescence) : '&
            ,min_leaf_age_for_senescence

       turn_count = 0

       !Config Key   = SS_BRANCH_TURN
       !Config Desc  = 
       !Config If    = 
       !Config Def   = 
       !Config Help  = 
       !Config Units = 
       !!!!!!!! needs a default value
       ss_branch_turn=un
       CALL getin_p  ('SS_BRANCH_TURN',ss_branch_turn)

       firstcall = .FALSE.


    ENDIF

    !! 2. Initializations 

    !! 2.1 set output to zero
    turnover(:,:,:,:) = zero
    new_turnover_time(:,:) = zero
    senescence(:,:) = .FALSE.
    sapwood_age(:,:)=un

    ! save old biomass
    bm_old(:,:,:,:) = biomass(:,:,:,:)
    ! Compute the turnover of branches 
    ! (2.5% per year : Orchidee standard, Masera 2003, Lehtonen 2004, DeAngelis 1981)
    branch_turn = 0.025/(one_year/dt)*ss_branch_turn

    !! 2.2 Recalculate mean leaf age
    !      Mean leaf age is recalculated from leaf ages of separate leaf age classes. Should actually be recalculated at the 
    !      end of this routine, but it does not change too fast.
    !      The mean leaf age is calculated using the following equation:
    !      \latexonly
    !      \input{turnover_lma_update_eqn1.tex}
    !      \endlatexonly
    !      \n
    leaf_meanage(:,:) = zero

    DO m = 1, nleafages
       leaf_meanage(:,:) = leaf_meanage(:,:) + leaf_age(:,:,m) * leaf_frac(:,:,m)
    ENDDO
    

    IF (ld_trnov) THEN
       WRITE(numout,*) 'leaf_meanage in turnover', leaf_meanage(:,test_pft)
       WRITE(numout,*) 'leaf_age in turnover', leaf_age(:,test_pft,:)
       WRITE(numout,*) 'leaf_frac in turnover', leaf_frac(:,test_pft,:)
    ENDIF
   
    CALL histwrite_p (hist_id_stomate, 'LEAFAGE', itime, &
           leaf_meanage, npts*nvm, horipft_index)
   

    !! 3. Climatic senescence

    !     Three different types of "climatic" leaf senescence,
    !     that do not change the age structure. 
    DO j = 2,nvm ! Loop over # PFTs

       !! 3.1 Determine if there is climatic senescence. 
       !      The climatic senescence can be of three types:
       !      sensitivity to cold temperatures, to lack of water, or both. If meteorological conditions are 
       !      fulfilled, a flag senescence is set to TRUE.
       !      Evergreen species do not experience climatic senescence.

       SELECT CASE ( senescence_type(j) )


       CASE ('crop' )!for crop senescence is based on a GDD criterium as in crop models
          WHERE ( ( biomass(:,j,ileaf,icarbon) .GT. zero ) .AND. &
               ( leaf_meanage(:,j) .GT. min_leaf_age_for_senescence(j) ) .AND. &
               ( gdd_from_growthinit(:,j) .GT.  gdd_senescence(j)))
             senescence(:,j) = .TRUE.
          ENDWHERE

       CASE ( 'cold' )

          !! 3.1.1 Summergreen species: Climatic senescence is triggered by sensitivity to cold temperatures
          !        Climatic senescence is triggered by sensitivity to cold temperatures as follows: 
          !        If biomass is large enough (i.e. when it is greater than zero), 
          !        AND (i.e. when leaf mean age is above a certain PFT-dependent treshold ::min_leaf_age_for_senescence,
          !        which is given in ::stomate_constants),      
          !        AND the monthly temperature is low enough (i.e. when monthly temperature ::t2m_month(:) is below a critical 
          !        temperature ::t_crit(:), which is calculated in this module),
          !        AND the temperature tendency is negative (i.e. when weekly temperatures ::t2m_week(:) are lower than monthly 
          !        temperatures ::t2m_month(:))
          !        If these conditions are met, senescence is set to TRUE.
          !
          !        The critical temperature for senescence is calculated using the following equation:
          !        \latexonly
          !        \input{turnover_temp_crit_eqn2.tex}
          !        \endlatexonly
          !        \n
          !
          ! Critical temperature for senescence may depend on long term annual mean temperature
          tl(:) = tlong_ref(:) - ZeroCelsius
          t_crit(:) = ZeroCelsius + senescence_temp(j,1) + &
               tl(:) * senescence_temp(j,2) + &
               tl(:)*tl(:) * senescence_temp(j,3)

          WHERE ( ( biomass(:,j,ileaf,icarbon) .GT. zero ) .AND. &
               ( leaf_meanage(:,j) .GT. min_leaf_age_for_senescence(j) ) .AND. &
               ( t2m_month(:) .LT. t_crit(:) ) .AND. ( t2m_week(:) .LT. t2m_month(:) ) )

             senescence(:,j) = .TRUE.

          ENDWHERE

       CASE ( 'dry' )

          !! 3.1.2 Raingreen species: Climatic senescence is triggered by sensitivity to lack of water availability 
          !        Climatic senescence is triggered by sensitivity to lack of water availability as follows:  
          !        If biomass is large enough (i.e. when it is greater than zero), 
          !        AND (i.e. when leaf mean age is above a certain PFT-dependent treshold ::min_leaf_age_for_senescence,
          !        which is given in ::stomate_constants),      
          !        AND the moisture availability drops below a critical level (i.e. when weekly moisture availability 
          !        ::moiavail_week(:,j) is below a critical moisture availability ::moiavail_crit(:),
          !        which is calculated in this module), 
          !        If these conditions are met, senescence is set to TRUE.
          !
          !        The moisture availability critical level is calculated using the following equation:
          !        \latexonly
          !        \input{turnover_moist_crit_eqn3.tex}
          !        \endlatexonly
          !        \n
          moiavail_crit(:) = &
               MIN( MAX( minmoiavail_lastyear(:,j) + hum_frac(j) * &
               ( maxmoiavail_lastyear(:,j) - minmoiavail_lastyear(:,j) ), &
               senescence_hum(j) ), &
               nosenescence_hum(j) )

          WHERE ( ( biomass(:,j,ileaf,icarbon) .GT. zero ) .AND. &
               ( leaf_meanage(:,j) .GT. min_leaf_age_for_senescence(j) ) .AND. &
               ( moiavail_week(:,j) .LT. moiavail_crit(:) ) )

             senescence(:,j) = .TRUE.

          ENDWHERE

       CASE ( 'mixed' )

          !! 3.1.3 Mixed criterion: Climatic senescence is triggered by sensitivity to temperature or to lack of water  
          !        Climatic senescence is triggered by sensitivity to temperature or to lack of water availability as follows:
          !        If biomass is large enough (i.e. when it is greater than zero), 
          !        AND (i.e. when leaf mean age is above a certain PFT-dependent treshold ::min_leaf_age_for_senescence,
          !        which is given in ::stomate_constants),      
          !        AND the moisture availability drops below a critical level (i.e. when weekly moisture availability 
          !        ::moiavail_week(:,j) is below a critical moisture availability ::moiavail_crit(:), calculated in this module), 
          !        OR 
          !        the monthly temperature is low enough (i.e. when monthly temperature ::t2m_month(:) is below a critical 
          !        temperature ::t_crit(:), calculated in this module),
          !        AND the temperature tendency is negative (i.e. when weekly temperatures ::t2m_week(:) are lower than 
          !        monthly temperatures ::t2m_month(:)).
          !        If these conditions are met, senescence is set to TRUE.
          moiavail_crit(:) = &
               MIN( MAX( minmoiavail_lastyear(:,j) + hum_frac(j) * &
               (maxmoiavail_lastyear(:,j) - minmoiavail_lastyear(:,j) ), &
               senescence_hum(j) ), &
               nosenescence_hum(j) )

          tl(:) = tlong_ref(:) - ZeroCelsius
          t_crit(:) = ZeroCelsius + senescence_temp(j,1) + &
               tl(:) * senescence_temp(j,2) + &
               tl(:)*tl(:) * senescence_temp(j,3)

          IF ( is_tree(j) ) THEN
             ! critical temperature for senescence may depend on long term annual mean temperature
             WHERE ( ( biomass(:,j,ileaf,icarbon) .GT. zero ) .AND. &
                  ( leaf_meanage(:,j) .GT. min_leaf_age_for_senescence(j) ) .AND. &
                  ( ( moiavail_week(:,j) .LT. moiavail_crit(:) ) .OR. &
                  ( ( t2m_month(:) .LT. t_crit(:) ) .AND. ( t2m_week(:) .LT. t2m_month(:) ) ) ) )
                senescence(:,j) = .TRUE.
             ENDWHERE

          ELSE
             
             ! The nparts dimension was introduced for the prognstic part. Use dimension 1 so that
             ! no new variable needs to be defined
             turnover_time(:,j,1) = max_turnover_time(j)
             
             WHERE ( ( biomass(:,j,ileaf,icarbon) .GT. zero ) .AND. &
                  ( leaf_meanage(:,j) .GT. min_leaf_age_for_senescence(j) ) .AND. &
                  ( ( moiavail_week(:,j) .LT. moiavail_crit(:) )))
                turnover_time(:,j,1) = max_turnover_time(j) * &
                     (1.-   (1.- (moiavail_week(:,j)/  moiavail_crit(:)))**2)
             ENDWHERE
             
             WHERE ( turnover_time(:,j,1) .LT. min_turnover_time(j) )
                turnover_time(:,j,1) = min_turnover_time(j)
             ENDWHERE
             
             WHERE ((( biomass(:,j,ileaf,icarbon) .GT. zero ) .AND. &
                  ( leaf_meanage(:,j) .GT. min_leaf_age_for_senescence(j) ) .AND. &
                  ((t2m_month(:) .LT. t_crit(:)) .AND. (lai(:,j) .GT. lai_max(j)/4.) .OR. &
                  (t2m_month(:) .LT. ZeroCelsius)) .AND. ( t2m_week(:) .LT. t2m_month(:) )))
                turnover_time(:,j,1)= leaffall(j) 
             ENDWHERE
             
          ENDIF


       !! Evergreen species do not experience climatic senescence
       CASE ( 'none' )

          
       !! In case no climatic senescence type is recognized.
       CASE default

          WRITE(numout,*) '  turnover: don''t know how to treat this PFT.'
          WRITE(numout,*) '  number (::j) : ',j
          WRITE(numout,*) '  senescence type (::senescence_type(j)) : ',senescence_type(j)

          STOP

       END SELECT

       !! 3.2 Drop leaves and roots, plus stems and fruits for grasses

       IF ( is_tree(j) ) THEN

          !! 3.2.1 Trees in climatic senescence lose their fine roots at the same rate as they lose their leaves. 
          !        The rate of biomass loss of both fine roots and leaves is presribed through the equation:
          !        \latexonly
          !        \input{turnover_clim_senes_biomass_eqn4.tex}
          !        \endlatexonly
          !        \n
          !         with ::leaffall(j) a PFT-dependent time constant which is given in ::stomate_constants),
          WHERE ( senescence(:,j) )

             turnover(:,j,ileaf,icarbon) = biomass(:,j,ileaf,icarbon) * dt / leaffall(j)
             turnover(:,j,iroot,icarbon) = biomass(:,j,iroot,icarbon) * dt / leaffall(j)

          ENDWHERE

       ELSE

          !! 3.2.2 In grasses, leaf senescence is extended to the whole plant 
          !        In grasses, leaf senescence is extended to the whole plant (all carbon pools) except to its
          !        carbohydrate reserve.      

          IF (senescence_type(j) .EQ. 'crop') THEN
             ! 3.2.2.1 crops with 'crop' phenological model
             WHERE ( senescence(:,j) )
                turnover(:,j,ileaf,icarbon) = biomass(:,j,ileaf,icarbon) * dt / leaffall(j)
                turnover(:,j,iroot,icarbon) = biomass(:,j,iroot,icarbon) * dt / leaffall(j)
                turnover(:,j,isapabove,icarbon) = biomass(:,j,isapabove,icarbon) * dt / leaffall(j)
                turnover(:,j,ifruit,icarbon) = biomass(:,j,ifruit,icarbon) * dt /leaffall(j)
             ENDWHERE
          ELSE

             ! 3.2.2.2 grass or crops based on 'mixed' phenological model
             WHERE (turnover_time(:,j,1) .LT. max_turnover_time(j)) 
                turnover(:,j,ileaf,icarbon) = biomass(:,j,ileaf,icarbon) * dt / turnover_time(:,j,1)
                turnover(:,j,isapabove,icarbon) = biomass(:,j,isapabove,icarbon) * dt / turnover_time(:,j,1)
                turnover(:,j,iroot,icarbon) = biomass(:,j,iroot,icarbon) * dt / turnover_time(:,j,1) 
                turnover(:,j,ifruit,icarbon) = biomass(:,j,ifruit,icarbon) * dt / turnover_time(:,j,1)
             ENDWHERE
          ENDIF
       ENDIF      ! tree/grass

       biomass(:,j,ileaf,icarbon) = biomass(:,j,ileaf,icarbon) - turnover(:,j,ileaf,icarbon)
       biomass(:,j,isapabove,icarbon) = biomass(:,j,isapabove,icarbon) - turnover(:,j,isapabove,icarbon)
       biomass(:,j,iroot,icarbon) = biomass(:,j,iroot,icarbon) - turnover(:,j,iroot,icarbon)
       biomass(:,j,ifruit,icarbon) = biomass(:,j,ifruit,icarbon) - turnover(:,j,ifruit,icarbon)

    ENDDO        ! loop over PFTs

    !! 4. Leaf fall
    !     At a certain age, leaves fall off, even if the climate would allow a green plant
    !     all year round. Even if the meteorological conditions are favorable for leaf maintenance,
    !     plants, and in particular, evergreen trees, have to renew their leaves simply because the 
    !     old leaves become inefficient.   
    !     Roots, fruits (and stems) follow leaves. The decay rate varies with leaf age.
    !     Note that plant is not declared senescent in this case (wchich is important for allocation:
    !     if the plant loses leaves because of their age, it can renew them).
    !
    !     The leaf turnover rate due to aging of leaves is calculated using the following equation:
    !     \latexonly
    !     \input{turnover_age_senes_biomass_eqn5.tex}
    !     \endlatexonly
    !     \n
    DO j = 2,nvm ! Loop over # PFTs

       !! save old leaf mass
       lm_old(:) = biomass(:,j,ileaf,icarbon)

       !! initialize leaf mass change in age class
       delta_lm(:,:) = zero

       IF ( is_tree(j) .OR. (.NOT. natural(j)) ) THEN

          !! 4.1 Trees: leaves, roots, fruits roots and fruits follow leaves.

          !! 4.1.1 Critical age: prescribed for trees
          leaf_age_crit(:,j) = tau_leaf(j)

       ELSE

          !! 4.2 Grasses: leaves, roots, fruits, sap follow leaves.

          !! 4.2.1 Critical leaf age depends on long-term temperature
          !        Critical leaf age depends on long-term temperature
          !        generally, lower turnover in cooler climates.
          leaf_age_crit(:,j) = &
               MIN( tau_leaf(j) * leaf_age_crit_coeff(1) , &
               MAX( tau_leaf(j) * leaf_age_crit_coeff(2) , &
               tau_leaf(j) - leaf_age_crit_coeff(3) * &
               ( tlong_ref(:)-ZeroCelsius - leaf_age_crit_tref ) ) )

       END IF
          
       ! 4.2.2 Loop over leaf age classes
       DO m = 1, nleafages

          turnover_rate(:) = zero

          WHERE ( leaf_age(:,j,m) .GT. leaf_age_crit(:,j)/2. )

             turnover_rate(:) =  &
                  MIN( 0.99_r_std, dt / ( leaf_age_crit(:,j) * &
                  ( leaf_age_crit(:,j) / leaf_age(:,j,m) )**quatre ) )

          ENDWHERE
          
          dturnover(:) = biomass(:,j,ileaf,icarbon) * leaf_frac(:,j,m) * turnover_rate(:)
          turnover(:,j,ileaf,icarbon) = turnover(:,j,ileaf,icarbon) + dturnover(:)
          biomass(:,j,ileaf,icarbon) = biomass(:,j,ileaf,icarbon) - dturnover(:)

          ! save leaf mass change
          delta_lm(:,m) = - dturnover(:)
          
          dturnover(:) = biomass(:,j,iroot,icarbon) * leaf_frac(:,j,m) * turnover_rate(:)
          turnover(:,j,iroot,icarbon) = turnover(:,j,iroot,icarbon) + dturnover(:)
          biomass(:,j,iroot,icarbon) = biomass(:,j,iroot,icarbon) - dturnover(:)
          
          dturnover(:) = biomass(:,j,ifruit,icarbon) * leaf_frac(:,j,m) * turnover_rate(:)
          turnover(:,j,ifruit,icarbon) = turnover(:,j,ifruit,icarbon) + dturnover(:)
          biomass(:,j,ifruit,icarbon) = biomass(:,j,ifruit,icarbon) - dturnover(:)
          
          IF (.NOT. is_tree(j)) THEN
             dturnover(:) = biomass(:,j,isapabove,icarbon) * leaf_frac(:,j,m) * turnover_rate(:)
             turnover(:,j,isapabove,icarbon) = turnover(:,j,isapabove,icarbon) + dturnover(:)
             biomass(:,j,isapabove,icarbon) = biomass(:,j,isapabove,icarbon) - dturnover(:)
          ENDIF
          
       ENDDO

       

       !! 4.3 Recalculate the fraction of leaf biomass in each leaf age class.
       !      Older leaves will fall more fast than younger leaves and therefore 
       !      the leaf age distribution needs to be recalculated after turnover. 
       !      The fraction of biomass in each leaf class is updated using the following equation:
       !      \latexonly
       !      \input{turnover_update_LeafAgeDistribution_eqn6.tex}
       !      \endlatexonly
       !      \n
       !
       !      new fraction = new leaf mass of that fraction / new total leaf mass
       !                   = (old fraction*old total leaf mass ::lm_old(:) + biomass change of that fraction ::delta_lm(:,m)  ) /
       !                     new total leaf mass ::biomass(:,j,ileaf
       DO m = 1, nleafages
          
          WHERE ( biomass(:,j,ileaf,icarbon) .GT. zero )

             leaf_frac(:,j,m) = ( leaf_frac(:,j,m)*lm_old(:) + delta_lm(:,m) ) / biomass(:,j,ileaf,icarbon)

          ELSEWHERE

             leaf_frac(:,j,m) = zero

          ENDWHERE
        
       ENDDO
       
    ENDDO         ! loop over PFTs

    !! 5. New (provisional) LAI 
    !     ::lai(:,j) is determined from the leaf biomass ::biomass(:,j,ileaf,icarbon) and the 
    !     specific leaf surface :: sla(j) (m^2 gC^{-1})
    !     The leaf area index is updated using the following equation:
    !     \latexonly
    !     \input{turnover_update_LAI_eqn7.tex}
    !     \endlatexonly
    !     \n

    !    lai(:,ibare_sechiba) = zero
    !    DO j = 2, nvm ! Loop over # PFTs
    !        lai(:,j) = biomass(:,j,ileaf,icarbon) * sla(j)
    !    ENDDO

    !! 6. Definitely drop all leaves if there is a very low leaf mass during senescence.

    !     Both for deciduous trees and grasses same conditions are checked:
    !     If biomass is large enough (i.e. when it is greater than zero), 
    !     AND when senescence is set to true
    !     AND the leaf biomass drops below a critical minimum biomass level (i.e. when it is lower than half
    !     the minimum initial LAI ::lai_initmin(j) divided by the specific leaf area ::sla(j),
    !     ::lai_initmin(j) is set to 0.3 in stomate_data.f90 and sla is a constant that is set to 0.015366 m2/gC), 
    !     If these conditions are met, the flag ::shed_rest(:) is set to TRUE.
    !
    !     After this, the biomass of different carbon pools both for trees and grasses is set to zero
    !     and the mean leaf age is reset to zero.
    !     Finally, the leaf fraction and leaf age of the different leaf age classes is set to zero.
    DO j = 2,nvm ! Loop over # PFTs

       shed_rest(:) = .FALSE.

       !! 6.1 For deciduous trees: next to leaves, also fruits and fine roots are dropped 
       !      For deciduous trees: next to leaves, also fruits and fine roots are dropped: fruit ::biomass(:,j,ifruit) 
       !      and fine root ::biomass(:,j,iroot) carbon pools are set to zero.
       IF ( is_tree(j) .AND. ( senescence_type(j) .NE. 'none' ) ) THEN

          ! check whether we shed the remaining leaves
          WHERE ( ( biomass(:,j,ileaf,icarbon) .GT. zero ) .AND. senescence(:,j) .AND. &
               ( biomass(:,j,ileaf,icarbon) .LT. (lai_initmin(j) / 2.)/sla(j) )             )

             shed_rest(:) = .TRUE.

             turnover(:,j,ileaf,icarbon)  = turnover(:,j,ileaf,icarbon) + biomass(:,j,ileaf,icarbon)
             turnover(:,j,iroot,icarbon)  = turnover(:,j,iroot,icarbon) + biomass(:,j,iroot,icarbon)
             turnover(:,j,ifruit,icarbon) = turnover(:,j,ifruit,icarbon) + biomass(:,j,ifruit,icarbon)

             biomass(:,j,ileaf,icarbon)  = zero
             biomass(:,j,iroot,icarbon)  = zero
             biomass(:,j,ifruit,icarbon) = zero

             ! reset leaf age
             leaf_meanage(:,j) = zero

          ENDWHERE

       ENDIF

       !! 6.2 For grasses: all aboveground carbon pools, except the carbohydrate reserves are affected: 
       !      For grasses: all aboveground carbon pools, except the carbohydrate reserves are affected: 
       !      fruit ::biomass(:,j,ifruit,icarbon), fine root ::biomass(:,j,iroot,icarbon) and sapwood above 
       !      ::biomass(:,j,isapabove,icarbon) carbon pools are set to zero. 
       IF ( .NOT. is_tree(j) ) THEN

          ! Shed the remaining leaves if LAI very low.
          WHERE ( ( biomass(:,j,ileaf,icarbon) .GT. zero ) .AND. senescence(:,j) .AND. &
               (  biomass(:,j,ileaf,icarbon) .LT. (lai_initmin(j) / 2.)/sla(j) ))

             shed_rest(:) = .TRUE.

             turnover(:,j,ileaf,icarbon) = turnover(:,j,ileaf,icarbon) + biomass(:,j,ileaf,icarbon)
             turnover(:,j,isapabove,icarbon) = turnover(:,j,isapabove,icarbon) + biomass(:,j,isapabove,icarbon)
             turnover(:,j,iroot,icarbon) = turnover(:,j,iroot,icarbon) + biomass(:,j,iroot,icarbon)
             turnover(:,j,ifruit,icarbon) = turnover(:,j,ifruit,icarbon) + biomass(:,j,ifruit,icarbon)

             biomass(:,j,ileaf,icarbon) = zero
             biomass(:,j,isapabove,icarbon) = zero
             biomass(:,j,iroot,icarbon) = zero
             biomass(:,j,ifruit,icarbon) = zero

             ! reset leaf age
             leaf_meanage(:,j) = zero

          ENDWHERE

       ENDIF

       !! 6.3 Reset the leaf age structure: the leaf fraction and leaf age of the different leaf age classes is set to zero.
      
       DO m = 1, nleafages

          WHERE ( shed_rest(:) )

             leaf_age(:,j,m) = zero
             leaf_frac(:,j,m) = zero

          ENDWHERE

       ENDDO



      !! 6.4 drop part of the branches and mortality of smaller trees until average height reaches 10 m 
      !! (herbivory, water shortage, ...). This "youth mortality" does not affect tree density as young 
      !! dead trees are immediately replaced.
      !
      IF ( control%forest_management ) THEN
          
          IF ( is_tree(j) ) THEN
             
             WHERE (forest_managed(:,j) > 0)

                ! Drop part of the branches
                turnover(:,j,isapabove,icarbon) = branch_ratio(j)*branch_turn*biomass(:,j,isapabove,icarbon)
                turnover(:,j,iheartabove,icarbon) = branch_ratio(j)*branch_turn*biomass(:,j,iheartabove,icarbon)
                turnover(:,j,isapbelow,icarbon) = branch_ratio(j)*branch_turn*biomass(:,j,isapbelow,icarbon)
                turnover(:,j,iheartbelow,icarbon) = branch_ratio(j)*branch_turn*biomass(:,j,iheartbelow,icarbon)

                biomass(:,j,isapabove,icarbon) = biomass(:,j,isapabove,icarbon) - turnover(:,j,isapabove,icarbon)
                biomass(:,j,iheartabove,icarbon) = biomass(:,j,iheartabove,icarbon) - turnover(:,j,iheartabove,icarbon)
                biomass(:,j,isapbelow,icarbon) = biomass(:,j,isapbelow,icarbon) - turnover(:,j,isapbelow,icarbon)
                biomass(:,j,iheartbelow,icarbon) = biomass(:,j,iheartbelow,icarbon) - turnover(:,j,iheartbelow,icarbon)

             END WHERE

          END IF

       END IF ! (control%forest_management)

    ENDDO          ! loop over PFTs
    
    !! 7. Herbivore activity: elephants, cows, gazelles but no lions.
 
    !     Herbivore activity affects the biomass of leaves and fruits as well 
    !     as stalks (only for grasses). Herbivore activity does not modify leaf 
    !     age structure. Herbivores ::herbivores(:,j) is the time constant of 
    !     probability of a leaf to be eaten by a herbivore, and is calculated in 
    !     ::stomate_season. following Mc Naughton et al. [1989].

    IF ( control%ok_herbivory ) THEN

       ! If the herbivore activity is allowed (if ::control%ok_herbivory is true, which is set in run.def), 
       ! remove the amount of biomass consumed by herbivory from the leaf biomass ::biomass(:,j,ileaf,icarbon) and 
       ! the fruit biomass ::biomass(:,j,ifruit,icarbon).
       ! The daily amount consumed equals the biomass multiplied by 1 day divided by the time constant ::herbivores(:,j).
       DO j = 2,nvm ! Loop over # PFTs

          IF ( is_tree(j) ) THEN

             !! For trees: only the leaves and fruit carbon pools are affected

             WHERE (biomass(:,j,ileaf,icarbon) .GT. zero)

                ! added by shilong
                WHERE (herbivores(:,j).GT. zero)

                   dturnover(:) = biomass(:,j,ileaf,icarbon) * dt / herbivores(:,j)
                   turnover(:,j,ileaf,icarbon) = turnover(:,j,ileaf,icarbon) + dturnover(:)
                   biomass(:,j,ileaf,icarbon) = biomass(:,j,ileaf,icarbon) - dturnover(:)

                   dturnover(:) = biomass(:,j,ifruit,icarbon) * dt / herbivores(:,j)
                   turnover(:,j,ifruit,icarbon) = turnover(:,j,ifruit,icarbon) + dturnover(:)
                   biomass(:,j,ifruit,icarbon) = biomass(:,j,ifruit,icarbon) - dturnover(:)

                ENDWHERE

             ENDWHERE

          ELSE

             ! For grasses: all aboveground carbon pools are affected: leaves, fruits and sapwood above
             WHERE ( biomass(:,j,ileaf,icarbon) .GT. zero )

                ! added by shilong
                WHERE (herbivores(:,j) .GT. zero)

                   dturnover(:) = biomass(:,j,ileaf,icarbon) * dt / herbivores(:,j)
                   turnover(:,j,ileaf,icarbon) = turnover(:,j,ileaf,icarbon) + dturnover(:)
                   biomass(:,j,ileaf,icarbon) = biomass(:,j,ileaf,icarbon) - dturnover(:)

                   dturnover(:) = biomass(:,j,isapabove,icarbon) * dt / herbivores(:,j)
                   turnover(:,j,isapabove,icarbon) = turnover(:,j,isapabove,icarbon) + dturnover(:)
                   biomass(:,j,isapabove,icarbon) = biomass(:,j,isapabove,icarbon) - dturnover(:)

                   dturnover(:) = biomass(:,j,ifruit,icarbon) * dt / herbivores(:,j)
                   turnover(:,j,ifruit,icarbon) = turnover(:,j,ifruit,icarbon) + dturnover(:)
                   biomass(:,j,ifruit,icarbon) = biomass(:,j,ifruit,icarbon) - dturnover(:)

                ENDWHERE

             ENDWHERE

          ENDIF  ! tree/grass?

       ENDDO    ! loop over PFTs

    ENDIF ! end herbivores

    !! 8. Fruit turnover for trees

    !     Fruit turnover for trees: trees simply lose their fruits with a time constant ::tau_fruit(j), 
    !     that is set to 90 days for all PFTs in ::stomate_constants

    DO k = 1,nelements 

       DO j = 2,nvm ! Loop over # PFTs

          IF ( is_tree(j) ) THEN

             dturnover(:) = biomass(:,j,ifruit,k) * dt / tau_fruit(j)
             turnover(:,j,ifruit,k) = turnover(:,j,ifruit,k) + dturnover(:)
             biomass(:,j,ifruit,k) = biomass(:,j,ifruit,k) - dturnover(:)
             
          ENDIF

       ENDDO       ! loop over PFTs
   
    END DO

    !! 9 Conversion of sapwood to heartwood both for aboveground and belowground sapwood and heartwood.

    !   Following LPJ (Sitch et al., 2003), sapwood biomass is converted into heartwood biomass 
    !   with a time constant tau ::tau_sap(j) of 1 year.
    !   Note that this biomass conversion is not added to "turnover" as the biomass is not lost!
    DO j = 2,nvm ! Loop over # PFTs

       IF ( is_tree(j) ) THEN

          !! For the recalculation of age in 9.2 (in case the vegetation is not dynamic ie. ::control%ok_dgvm is FALSE), 
          !! the heartwood above and below is stored in ::hw_old(:).
          IF ( .NOT. control%ok_dgvm ) THEN

             hw_old(:) = biomass(:,j,iheartabove,icarbon) + biomass(:,j,iheartbelow,icarbon)
             sw_old(:) = biomass(:,j,isapabove,icarbon) + biomass(:,j,isapbelow,icarbon)

          ENDIF

          !! 9.1 Calculate the rate of sapwood to heartwood conversion 
          !      Calculate the rate of sapwood to heartwood conversion with the time constant ::tau_sap(j) 
          !      and update aboveground and belowground sapwood ::biomass(:,j,isapabove) and ::biomass(:,j,isapbelow)
          !      and heartwood ::biomass(:,j,iheartabove) and ::biomass(:,j,iheartbelow).

          DO k = 1,nelements

             ! Above the ground
             sapconv(:) = biomass(:,j,isapabove,k) * dt / tau_sap(j)
             biomass(:,j,isapabove,k) = biomass(:,j,isapabove,k) - sapconv(:)
             biomass(:,j,iheartabove,k) =  biomass(:,j,iheartabove,k) + sapconv(:)
             
             ! Below the ground
             sapconv(:) = biomass(:,j,isapbelow,k) * dt / tau_sap(j)
             biomass(:,j,isapbelow,k) = biomass(:,j,isapbelow,k) - sapconv(:)
             biomass(:,j,iheartbelow,k) =  biomass(:,j,iheartbelow,k) + sapconv(:)

          END DO

          !! 9.2 If the vegetation is not dynamic, the age of the plant is decreased. 
          !      The updated heartwood, the sum of new heartwood above and new heartwood below after 
          !      converting sapwood to heartwood, is saved as ::hw_new(:) .
          !      Creation of new heartwood decreases the age of the plant with a factor that is determined by: 
          !      old heartwood ::hw_old(:) divided by the new heartwood ::hw_new(:)
          IF ( .NOT. control%ok_dgvm ) THEN

             hw_new(:) = biomass(:,j,iheartabove,icarbon) + biomass(:,j,iheartbelow,icarbon)
             sw_new(:) = biomass(:,j,isapabove,icarbon) + biomass(:,j,isapbelow,icarbon)

             WHERE ( hw_new(:) .GT. zero )

                age(:,j) = age(:,j) * hw_old(:)/hw_new(:)
                sapwood_age(:,j) = sapwood_age(:,j) * sw_old(:)/sw_new(:)

             ENDWHERE

          ENDIF

       ENDIF

    ENDDO       ! loop over PFTs


    CALL xios_orchidee_send_field("HERBIVORES",herbivores)
    CALL xios_orchidee_send_field("LEAF_AGE",leaf_meanage)
    

    ! Write mean leaf age and time constant of probability of a leaf to be eaten by a herbivore 
    ! to the stomate output file.
    !CALL histwrite_p (hist_id_stomate, 'LEAFAGE', itime, &
    !     leaf_meanage, npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'HERBIVORES', itime, &
         herbivores, npts*nvm, horipft_index)

    IF (bavard.GE.4) WRITE(numout,*) 'Leaving turnover'

  END SUBROUTINE turnover_diagnostic



!! ================================================================================================================================
!! SUBROUTINE    : turnover_prognostic
!!
!>\BRIEF         Calculate turnover of leaves, roots, fruits and sapwood due to aging or climatic 
!!               induced senescence. Calculate herbivory.
!!
!! DESCRIPTION : This subroutine determines the turnover of leaves and fine roots (and stems for grasses)
!! and simulates following processes:
!! 1. Mean leaf age is calculated from leaf ages of separate leaf age classes. Should actually 
!!    be recalculated at the end of the routine, but it does not change too fast. The mean leaf 
!!    age is calculated using the following equation:
!!    \latexonly
!!    \input{turnover_lma_update_eqn1.tex}
!!    \endlatexonly
!!    \n 
!! 2. Meteorological senescence: the detection of the end of the growing season and shedding 
!!    of leaves, fruits and fine roots due to unfavourable meteorological conditions.
!!    The model distinguishes three different types of "climatic" leaf senescence, that do not 
!!    change the age structure: sensitivity to cold temperatures, to lack of water, or both. 
!!    If meteorological conditions are fulfilled, a flag ::senescence is set to TRUE. Note
!!    that evergreen species do not experience climatic senescence.
!!    Climatic senescence is triggered by sensitivity to cold temperatures where the critical 
!!    temperature for senescence is calculated using the following equation:
!!    \latexonly
!!    \input{turnover_temp_crit_eqn2.tex}
!!    \endlatexonly
!!    \n
!!    Climatic senescence is triggered by sensitivity to lack of water availability where the 
!!    moisture availability critical level is calculated using the following equation:
!!    \latexonly
!!    \input{turnover_moist_crit_eqn3.tex}
!!    \endlatexonly
!!    \n
!!    Climatic senescence is triggered by sensitivity to temperature or to lack of water where
!!    critical temperature and moisture availability are calculated as above.\n
!!    Trees in climatic senescence lose their fine roots at the same rate as they lose their leaves. 
!!    The rate of biomass loss of both fine roots and leaves is presribed through the equation:
!!    \latexonly
!!    \input{turnover_clim_senes_biomass_eqn4.tex}
!!    \endlatexonly
!!    \n
!!    with ::leaffall(j) a PFT-dependent time constant which is given in 
!!    ::stomate_constants. In grasses, leaf senescence is extended to the whole plant 
!!    (all carbon pools) except to its carbohydrate reserve.    
!! 3. Senescence due to aging: the loss of leaves, fruits and  biomass due to aging
!!    At a certain age, leaves fall off, even if the climate would allow a green plant
!!    all year round. Even if the meteorological conditions are favorable for leaf maintenance,
!!    plants, and in particular, evergreen trees, have to renew their leaves simply because the 
!!    old leaves become inefficient. Roots, fruits (and stems for grasses) follow leaves. 
!!    The ??senescence?? rate varies with leaf age. Note that plant is not declared senescent 
!!    in this case (wchich is important for allocation: if the plant loses leaves because of 
!!    their age, it can renew them). The leaf turnover rate due to aging of leaves is calculated
!!    using the following equation:
!!    \latexonly
!!    \input{turnover_age_senes_biomass_eqn5.tex}
!!    \endlatexonly
!!    \n
!!    Drop all leaves if there is a very low leaf mass during senescence. After this, the biomass 
!!    of different carbon pools both for trees and grasses is set to zero and the mean leaf age 
!!    is reset to zero. Finally, the leaf fraction and leaf age of the different leaf age classes 
!!    is set to zero. For deciduous trees: next to leaves, also fruits and fine roots are dropped.
!!    For grasses: all aboveground carbon pools, except the carbohydrate reserves are affected:
!! 4. Update the leaf biomass, leaf age class fraction and the LAI
!!    Older leaves will fall more frequently than younger leaves and therefore the leaf age 
!!    distribution needs to be recalculated after turnover. The fraction of biomass in each 
!!    leaf class is updated using the following equation:
!!    \latexonly
!!    \input{turnover_update_LeafAgeDistribution_eqn6.tex}
!!    \endlatexonly
!!    \n
!! 5. Simulate herbivory activity and update leaf and fruits biomass. Herbivore activity 
!!    affects the biomass of leaves and fruits as well as stalks (only for grasses).
!!    However, herbivores do not modify leaf age structure.
!! 6. Calculates fruit turnover for trees. Trees simply lose their fruits with a time 
!!    constant ::tau_fruit(j), that is set to 90 days for all PFTs in ::stomate_constants 
!! 7. Convert sapwood to heartwood for trees and update heart and softwood above and 
!!    belowground biomass. Sapwood biomass is converted into heartwood biomass 
!!    with a time constant tau ::tau_sap(j) of 1 year. Note that this biomass conversion 
!!    is not added to "turnover" as the biomass is not lost. For the updated heartwood, 
!!    the sum of new heartwood above and new heartwood below after converting sapwood to 
!!    heartwood, is saved as ::hw_new(:). Creation of new heartwood decreases the age of 
!!    the plant ??carbon?? with a factor that is determined by: old heartwood ::hw_old(:) 
!!    divided by the new heartwood ::hw_new(:)
!!
!! RECENT CHANGE(S) : None
!!
!! MAIN OUTPUT VARIABLES: ::Biomass of leaves, fruits, fine roots and sapwood above (latter for grasses only), 
!!                        ::Update LAI, ::Update leaf age distribution with new leaf age class fraction 
!!
!! REFERENCE(S) : 
!! - Krinner, G., N. Viovy, N. de Noblet-Ducoudre, J. Ogee, J. Polcher, P. 
!! Friedlingstein, P. Ciais, S. Sitch and I.C. Prentice (2005), A dynamic global
!! vegetation model for studies of the coupled atmosphere-biosphere system, Global
!! Biogeochemical Cycles, 19, doi:10.1029/2003GB002199. 
!! - McNaughton, S. J., M. Oesterheld, D. A. Frank and K. J. Williams (1989), 
!! Ecosystem-level patterns of primary productivity and herbivory in terrestrial habitats, 
!! Nature, 341, 142-144, 1989. 
!! - Sitch, S., C. Huntingford, N. Gedney, P. E. Levy, M. Lomas, S. L. Piao, , Betts, R., Ciais, P., Cox, P., 
!! Friedlingstein, P., Jones, C. D., Prentice, I. C. and F. I. Woodward : Evaluation of the terrestrial carbon  
!! cycle, future plant geography and climate-carbon cycle feedbacks using 5 dynamic global vegetation 
!! models (dgvms), Global Change Biology, 14(9), 2015–2039, 2008. 
!!
!! FLOWCHART    : 
!! \latexonly
!! \includegraphics[scale=0.5]{turnover_flowchart_1.png}
!! \includegraphics[scale=0.5]{turnover_flowchart_2.png}
!! \endlatexonly
!! \n
!_ ================================================================================================================================

  SUBROUTINE turnover_prognostic (npts, dt, PFTpresent, herbivores, &
       maxmoiavail_lastyear, minmoiavail_lastyear, moiavail_week, moiavail_month, &
       tlong_ref, t2m_longterm, t2m_month, t2m_week, veget_max, &
       gdd_from_growthinit, leaf_age, leaf_frac, age, &
       biomass, turnover, senescence, turnover_time, &
       circ_class_biomass, circ_class_n, ind, allow_initpheno, &
       when_growthinit, tau_eff_leaf, tau_eff_sap, tau_eff_root, &
       harvest_pool, harvest_type, harvest_cut, harvest_area, &
       resolution, wstress_month)

 !! 0. Variable and parameter declaration

    !! 0.1 Input variables 

    INTEGER(i_std), INTENT(in)                       :: npts                 !! Domain size - number of grid cells 
                                                                             !! (unitless) 
    REAL(r_std), INTENT(in)                          :: dt                   !! time step (dt_days)
    LOGICAL, DIMENSION(:,:), INTENT(in)              :: PFTpresent           !! PFT exists (true/false)
    REAL(r_std),DIMENSION(:,:), INTENT(in)           :: resolution           !! The length in m of each grid-box in X and Y
    REAL(r_std), DIMENSION(:,:), INTENT(in)          :: herbivores           !! time constant of probability of a leaf to 
                                                                             !! be eaten by a herbivore (days) 
    REAL(r_std), DIMENSION(:,:), INTENT(in)          :: maxmoiavail_lastyear !! last year's maximum moisture availability 
                                                                             !! (0-1, unitless)
    REAL(r_std), DIMENSION(:,:), INTENT(in)          :: minmoiavail_lastyear !! last year's minimum moisture availability 
                                                                             !! (0-1, unitless)
    REAL(r_std), DIMENSION(:,:), INTENT(in)          :: moiavail_week        !! "weekly" moisture availability 
                                                                             !! (0-1, unitless)
    REAL(r_std), DIMENSION(:,:), INTENT(in)          :: moiavail_month       !! "monthly" moisture availability 
                                                                             !! (0-1, unitless)
    REAL(r_std), DIMENSION(:), INTENT(in)            :: tlong_ref            !! "long term" 2 meter reference 
                                                                             !! temperatures (K)
    REAL(r_std), DIMENSION(:), INTENT(in)            :: t2m_longterm         !! "longterm" 2-meter temperatures (K)
    REAL(r_std), DIMENSION(:), INTENT(in)            :: t2m_month            !! "monthly" 2-meter temperatures (K)
    REAL(r_std), DIMENSION(:), INTENT(in)            :: t2m_week             !! "weekly" 2 meter temperatures (K)
    REAL(r_std), DIMENSION(:,:), INTENT(in)          :: veget_max            !! "maximal" coverage fraction of a PFT (LAI 
                                                                             !! -> infinity) on ground (unitless) 
    REAL(r_std), DIMENSION(:,:), INTENT(in)          :: gdd_from_growthinit  !! gdd senescence for crop
    REAL(r_std), DIMENSION(:,:), INTENT(in)          :: when_growthinit      !! How many days ago was the beginning of 
                                                                             !! the growing season (days) 
    REAL(r_std), DIMENSION(:,:,:), INTENT(in)        :: circ_class_n         !! Number of individuals in each circ class
                                                                             !! @tex $(m^{-2})$ @endtex
    LOGICAL, DIMENSION(:,:), INTENT(in)              :: allow_initpheno      !! Are we allowed to declare the 
                                                                             !! beginning of the growing season?
                                                                             !! and the end of senescence (true/false) 
    REAL(r_std), DIMENSION(:,:), INTENT(in)          :: tau_eff_root         !! Effective root turnover time that accounts
                                                                             !! waterstress (days)
    REAL(r_std), DIMENSION(:,:), INTENT(in)          :: tau_eff_sap          !! Effective sapwood turnover time that accounts
                                                                             !! waterstress (days)
    REAL(r_std), DIMENSION(:,:), INTENT(in)          :: tau_eff_leaf         !! Effective leaf turnover time that accounts
                                                                             !! waterstress (days)
    REAL(r_std), DIMENSION(:,:), INTENT(in)          :: wstress_month        !! Water stress factor, based on hum_rel_daily
                                                                             !! (unitless, 0-1)


    !! 0.2 Output variables
   

    !! 0.3 Modified variables

    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)     :: leaf_age             !! age of the leaves (days)
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)     :: leaf_frac            !! fraction of leaves in leaf age class 
                                                                             !! (0-1, unitless)
    REAL(r_std), DIMENSION(:,:,:,:), INTENT(inout)   :: harvest_pool         !! The wood and biomass that have been
                                                                             !! havested by humans @tex $(gC)$ @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: harvest_type         !! Type of management that resulted
                                                                             !! in the harvest (unitless)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: harvest_cut          !! Type of cutting that was used for the harvest
                                                                             !! (unitless)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: harvest_area         !! Harvested area (m^{2})
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: age                  !! age (years)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: ind                  !! Density of individuals 
                                                                             !! @tex $(m^{-2})$ @endtex
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)     :: turnover_time        !! turnover_time of grasses (days) 
    REAL(r_std), DIMENSION(:,:,:,:), INTENT(inout)   :: biomass              !! biomass @tex ($gC m^{-2}$) @endtex
    REAL(r_std), DIMENSION(:,:,:,:), INTENT(inout)   :: turnover             !! Turnover @tex ($gC m^{-2}$) @endtex
    REAL(r_std), DIMENSION(:,:,:,:,:), INTENT(inout) :: circ_class_biomass   !! Biomass of the componets of the model  
                                                                             !! tree within a circumference
                                                                             !! class @tex $(gC ind^{-1})$ @endtex
    LOGICAL, DIMENSION(:,:), INTENT(inout)           :: senescence           !! is the plant senescent? (true/false) 
                                                                             !! (interesting only for deciduous trees: 
                                                                             !! carbohydrate reserve)
    
    !! 0.4 Local  variables

    LOGICAL, DIMENSION(npts)                         :: shed_rest            !! shed the remaining leaves? (true/false)
    INTEGER(i_std)                                   :: ivm,iele,ilage,ipts  !! Index (unitless)
    INTEGER(i_std)                                   :: ipar,icirc,imbc      !! Index (unitless)
    INTEGER(i_std), SAVE                             :: turn_count           !!  
!$OMP THREADPRIVATE(turn_count)
    REAL(r_std), DIMENSION(npts,nvm)                 :: leaf_meanage         !! mean age of the leaves (days)
    REAL(r_std), DIMENSION(npts,nvm)                 :: lai                  !! leaf area index @tex ($m^2 m^{-2}$) 
                                                                             !! @endtex
    REAL(r_std), DIMENSION(npts)                     :: dturnover            !! Intermediate variable for turnover ??
                                                                             !! @tex ($gC m^{-2}$) @endtex 
    REAL(r_std), DIMENSION(npts)                     :: moiavail_crit        !! critical moisture availability, function 
                                                                             !! of last year's moisture availability 
                                                                             !! (0-1, unitless)
    REAL(r_std), DIMENSION(npts)                     :: tl                   !! long term annual mean temperature, (C)
    REAL(r_std), DIMENSION(npts)                     :: t_crit               !! critical senescence temperature, function 
                                                                             !! of long term annual temperature (K) 
    REAL(r_std), DIMENSION(npts)                     :: sapconv              !! Sapwood conversion @tex ($gC m^{-2}$) 
                                                                             !! @endtex 
    REAL(r_std), DIMENSION(npts)                     :: hw_old               !! old heartwood mass @tex ($gC m^{-2}$) 
                                                                             !! @endtex 
    REAL(r_std), DIMENSION(npts)                     :: hw_new               !! new heartwood mass @tex ($gC m^{-2}$) 
                                                                             !! @endtex 
    REAL(r_std), DIMENSION(npts)                     :: lm_old               !! old leaf mass @tex ($gC m^{-2}$) @endtex
    REAL(r_std), DIMENSION(npts,nleafages)           :: delta_lm             !! leaf mass change for each age class @tex 
                                                                             !! ($gC m^{-2}$) @endtex 
    REAL(r_std), DIMENSION(npts)                     :: turnover_rate        !! turnover rate (unitless) 
    REAL(r_std), DIMENSION(npts,nvm)                 :: leaf_age_crit        !! critical leaf age (days)
    REAL(r_std), DIMENSION(npts,nvm)                 :: new_turnover_time    !! instantaneous turnover time (days)
    REAL(r_std), DIMENSION(npts,nvm)                 :: harvest_time         !! Prescribed harvest time adjusted for the
                                                                             !! longterm temperature at the pixel (days)
    REAL(r_std)                                      :: branch_turn          !! turnover of branches (how much goes 
                                                                             !! to litter each day) (cf article CO2fix)
    REAL(r_std), SAVE                                :: ss_branch_turn       !! Variations for sensitivity analysis
!$OMP THREADPRIVATE(ss_branch_turn)
    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements):: bm_old               !! old root mass
    REAL(r_std), DIMENSION(npts,nvm)                 :: sapwood_age          !! sapwood age (days)
    REAL(r_std), DIMENSION(npts)                     :: sw_old               !! old sapwood mass 
                                                                             !! (gC/(m**2 of nat/agri ground))
    REAL(r_std), DIMENSION(npts)                     :: sw_new               !! new sapwood mass 
                                                                             !! (gC/(m**2 of nat/agri ground)
    REAL(r_std), DIMENSION(npts)                     :: pixel_area           !! surface area of the pixel @tex ($m^{2}$) @endtex
    REAL(r_std), DIMENSION(npts,nvm,nmbcomp,nelements)&
                                                     :: check_intern         !! Contains the components of the internal
                                                                             !! mass balance chech for this routine
                                                                             !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nelements)       :: closure_intern       !! Check closure of internal mass balance
                                                                             !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nelements)       :: pool_start, pool_end !! Start and end pool of this routine 
                                                                             !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    
!_ ================================================================================================================================

    IF (bavard.GE.3) WRITE(numout,*) 'Entering prognostic turnover'

 !! 1. first call - output messages

    IF ( firstcall ) THEN

       WRITE(numout,*) 'turnover:'

       WRITE(numout,*) ' > minimum mean leaf age for senescence (days) (::min_leaf_age_for_senescence) : '&
            ,min_leaf_age_for_senescence

       turn_count = 0

       !Config Key   = SS_BRANCH_TURN
       !Config Desc  = 
       !Config If    = 
       !Config Def   = 
       !Config Help  = 
       !Config Units = 

       ! this needs to be initialized, else it crashes in the getin_p call
       ss_branch_turn=un
       CALL getin_p  ('SS_BRANCH_TURN',ss_branch_turn)

       firstcall = .FALSE.

    ENDIF


 !! 2. Initializations 

    !! 2.1 set output to zero
    turnover(:,:,:,:) = zero
    new_turnover_time(:,:) = zero
    harvest_time(:,:) = zero 

    !! 2.2 Initialize check for mass balance closure
    !  The mass balance is calculated at the end of this routine
    !  in section 10.
    ! turnover is always equal to zero, so maybe that term doesn't need
    ! to be included here.
    pool_start = zero
    DO ipar = 1,nparts
       DO iele = 1,nelements
          !  Initial biomass pool
          pool_start(:,:,iele) = pool_start(:,:,iele) + &
               (biomass(:,:,ipar,iele) * veget_max(:,:)) + &
               (turnover(:,:,ipar,iele) * veget_max(:,:))
       ENDDO
    ENDDO

    !  Surface area (m2) of the pixel
    pixel_area(:) = resolution(:,1) * resolution(:,2)

    ! The biomass harvest pool is expressed in gC pixel-1 So, it 
    ! shouldn't be multiplied by veget_max but it should be divided
    ! by pixel_area to obtain gC m-2.
    DO ivm = 1,nvm
       pool_start(:,ivm,icarbon) = pool_start(:,ivm,icarbon) + &
            SUM(harvest_pool(:,ivm,:,icarbon),2) / &
            pixel_area(:)
    ENDDO

    ! save old biomass
    bm_old(:,:,:,:) = biomass(:,:,:,:)

    ! Compute the turnover of branches 
    ! (2.5% per year : Orchidee standard, Masera 2003, Lehtonen 2004, DeAngelis 1981)
    branch_turn = 0.025/(one_year/dt)*ss_branch_turn

    ! Calculate lai
    DO ivm = 2,nvm
       DO ipts=1,npts
          lai(ipts,ivm) = biomass_to_lai(biomass(ipts,ivm,ileaf,icarbon),ivm)
       ENDDO
    ENDDO
    lai(:,1) = zero

    !! 2.2 Recalculate mean leaf age
    !  Mean leaf age is recalculated from leaf ages of separate leaf age classes. 
    !  Leaf age is used as a criterion in several of the following IF/WHERE loops 
    !  The mean leaf age is calculated using the following equation:
    !  \latexonly
    !  \input{turnover_lma_update_eqn1.tex}
    !  \endlatexonly
    !  \n
    leaf_meanage(:,:) = zero

    DO ilage = 1, nleafages

       leaf_meanage(:,:) = leaf_meanage(:,:) + leaf_age(:,:,ilage) * leaf_frac(:,:,ilage)

    ENDDO

 !! 3. Climatic senescence

    ! Three different types of "climatic" leaf senescence,
    ! that do not change the age structure. 
    DO ivm = 2,nvm ! Loop over # PFTs

       !! 3.1 Determine if there is climatic senescence. 
       !  The climatic senescence can be of three types: sensitivity to cold temperatures, 
       !  to lack of water, or both. If meteorological conditions are fulfilled, a flag 
       !  senescence is set to TRUE. Evergreen species do not experience climatic senescence.
       SELECT CASE ( senescence_type(ivm) )

       !! 3.1.1 Crops
       !  Crop senescence is based on a GDD criterium as in crop models.
       CASE ('crop' )
          
          !+++CHECK+++
          ! This is pure plumbing under time pressure (says the one who introduced it).     
          ! Switch senescence to true and it will stay true until begin_leaves becomes true
          ! A growing season should last about 4 months  where the longterm temperature 
          ! is 282 K (crude observation from N. France) before senescence can set in unless 
          ! the growing degrees are reached. In colder and warmer places the growing season
          ! is shortened or lengthened. Because in warmer places the growing season lasts 
          ! longer, crop can be planted that required more light.
          harvest_time(:,ivm) = 130 + 3 * ( t2m_longterm(:) - 282)

          WHERE ( ( biomass(:,ivm,ileaf,icarbon) .GT. zero ) .AND. &
               ( leaf_meanage(:,ivm) .GT. min_leaf_age_for_senescence(ivm) ) .AND. &
               ( ( gdd_from_growthinit(:,ivm) .GT. gdd_senescence(ivm) ) .OR. &
               ( ( when_growthinit(:,ivm) .GT. harvest_time(:,ivm) ) ) ) )

             senescence(:,ivm) = .TRUE.

          ENDWHERE
          !+++++++++++


       !! 3.1.2 Summergreen species
       !  Climatic senescence is triggered by sensitivity to cold temperatures as follows: 
       !  If biomass is large enough (i.e. when it is greater than zero), 
       !  AND (i.e. when leaf mean age is above a certain PFT-dependent treshold ::min_leaf_age_for_senescence,
       !  which is given in constants),      
       !  AND the monthly temperature is low enough (i.e. when monthly temperature ::t2m_month(:) is below a critical 
       !  temperature ::t_crit(:), which is calculated in this module),
       !  AND the temperature tendency is negative (i.e. when weekly temperatures ::t2m_week(:) are lower than monthly 
       !  temperatures ::t2m_month(:))
       !  If these conditions are met, senescence is set to TRUE.
       !
       !  The critical temperature for senescence is calculated using the following equation:
       !  \latexonly
       !  \input{turnover_temp_crit_eqn2.tex}
       !  \endlatexonly
       !  \n
       !  Critical temperature for senescence may depend on long term annual mean temperature
       CASE ( 'cold' )
 
          tl(:) = tlong_ref(:) - ZeroCelsius
          t_crit(:) = ZeroCelsius + senescence_temp(ivm,1) + &
               tl(:) * senescence_temp(ivm,2) + &
               tl(:)*tl(:) * senescence_temp(ivm,3)

          WHERE ( ( biomass(:,ivm,ileaf,icarbon) .GT. zero ) .AND. &
               ( leaf_meanage(:,ivm) .GT. min_leaf_age_for_senescence(ivm) ) .AND. &
               ( t2m_month(:) .LT. t_crit(:) ) .AND. &
               ( t2m_week(:) .LT. t2m_month(:) ) )

             senescence(:,ivm) = .TRUE.
          
          ENDWHERE

       !! 3.1.3 Raingreen species
       !  Climatic senescence is triggered by sensitivity to lack of water availability as follows:  
       !  If biomass is large enough (i.e. when it is greater than zero), 
       !  AND (i.e. when leaf mean age is above a certain PFT-dependent treshold ::min_leaf_age_for_senescence,
       !  which is given in ::stomate_constants),      
       !  AND the moisture availability drops below a critical level (i.e. when weekly moisture availability 
       !  ::moiavail_week(:,ivm) is below a critical moisture availability ::moiavail_crit(:),
       !  which is calculated in this module), 
       !  If these conditions are met, senescence is set to TRUE.
       !
       !  The moisture availability critical level is calculated using the following equation:
       !  \latexonly
       !  \input{turnover_moist_crit_eqn3.tex}
       !  \endlatexonly
       !  \n
       CASE ( 'dry' )

          moiavail_crit(:) = &
               MIN( MAX( minmoiavail_lastyear(:,ivm) + hum_frac(ivm) * &
               ( maxmoiavail_lastyear(:,ivm) - minmoiavail_lastyear(:,ivm) ), &
               senescence_hum(ivm) ), &
               nosenescence_hum(ivm) )

          WHERE ( ( biomass(:,ivm,ileaf,icarbon) .GT. zero ) .AND. &
               ( leaf_meanage(:,ivm) .GT. min_leaf_age_for_senescence(ivm) ) .AND. &
               ( moiavail_week(:,ivm) .LT. moiavail_crit(:) ) )

             senescence(:,ivm) = .TRUE.

          ENDWHERE

       !! 3.1.4 Mixed criterion: Climatic senescence is triggered by sensitivity to temperature or to lack of water  
       !  Climatic senescence is triggered by sensitivity to temperature or to lack of water availability as follows:
       !  If biomass is large enough (i.e. when it is greater than zero), 
       !  AND (i.e. when leaf mean age is above a certain PFT-dependent treshold ::min_leaf_age_for_senescence,
       !  which is given in ::stomate_constants),      
       !  AND the moisture availability drops below a critical level (i.e. when weekly moisture availability 
       !  ::moiavail_week(:,ivm) is below a critical moisture availability ::moiavail_crit(:), calculated in this module), 
       !  OR 
       !  the monthly temperature is low enough (i.e. when monthly temperature ::t2m_month(:) is below a critical 
       !  temperature ::t_crit(:), calculated in this module),
       !  AND the temperature tendency is negative (i.e. when weekly temperatures ::t2m_week(:) are lower than 
       !  monthly temperatures ::t2m_month(:)).
       !  If these conditions are met, senescence is set to TRUE.
       CASE ( 'mixed' )
    
          moiavail_crit(:) = MIN( MAX( minmoiavail_lastyear(:,ivm) + hum_frac(ivm) * &
               (maxmoiavail_lastyear(:,ivm) - minmoiavail_lastyear(:,ivm) ), &
               senescence_hum(ivm) ), nosenescence_hum(ivm) )
          tl(:) = tlong_ref(:) - ZeroCelsius
          t_crit(:) = ZeroCelsius + senescence_temp(ivm,1) + &
               tl(:) * senescence_temp(ivm,2) + &
               tl(:)*tl(:) * senescence_temp(ivm,3)

          IF ( is_tree(ivm) ) THEN

             ! critical temperature for senescence may depend on long term annual mean temperature
             WHERE ( ( biomass(:,ivm,ileaf,icarbon) .GT. zero ) .AND. &
                  ( leaf_meanage(:,ivm) .GT. min_leaf_age_for_senescence(ivm) ) .AND. &
                  ( ( moiavail_week(:,ivm) .LT. moiavail_crit(:) ) .OR. &
                  ( ( t2m_month(:) .LT. t_crit(:) ) .AND. ( t2m_week(:) .LT. t2m_month(:) ) ) ) )

                senescence(:,ivm) = .TRUE.

             ENDWHERE

          ! Grasses
          ELSE

!!$            !+++CHECK+++
!!$            ! There is no longer this kind of turnover outside the senescence period.
!!$            ! The physiological basis of this turnover is unclear because the turn
!!$            ! over due to ageing has been accounted for in §4. Not sure whether the
!!$            ! values for ::turnover_time calculated here are ever used because turnover
!!$            ! from ageing in §4 recalculates the turnover times. 
!!$            WHERE ( ( biomass(:,ivm,ileaf,icarbon) .GT. zero ) .AND. &
!!$                 ( leaf_meanage(:,ivm) .GT. min_leaf_age_for_senescence(ivm) ) .AND. &
!!$                 ( ( moiavail_week(:,ivm) .LT. moiavail_crit(:) )))
!!$
!!$               turnover_time(:,ivm,ileaf) = tau_eff_leaf(:,ivm) * &
!!$                    (un - (un - (moiavail_week(:,ivm)/  moiavail_crit(:)))**deux)
!!$               turnover_time(:,ivm,iroot) = tau_eff_root(:,ivm) * &
!!$                    (un - (un - (moiavail_week(:,ivm)/  moiavail_crit(:)))**deux)
!!$               turnover_time(:,ivm,isapabove) = tau_eff_sap(:,ivm) * &
!!$                    (un - (un - (moiavail_week(:,ivm)/  moiavail_crit(:)))**deux)
!!$               turnover_time(:,ivm,ifruit) = tau_fruit(ivm) * &
!!$                    (un - (un - (moiavail_week(:,ivm)/  moiavail_crit(:)))**deux)
!!$		
!!$            ENDWHERE
!!$            !+++++++++++

            ! Because the DOFOCO increase of LAI is much slower than in the trunk there
            ! is a period at the start of the growing season where LAI is less than lai_happy
            ! In that period t2_month is still increasing from its winter values but could be
            ! below t_crit. That condition resulted in many grasslands along the Atlantic and
            ! North Sea coast to go into senescence in May. If someone wants to put it back in
            ! be carefull with the brackets.((t2m_month(:) .LT. t_crit(:)) .AND. (lai(:,ivm) 
            ! .GT. lai_happy(ivm)))
            WHERE ( ( (biomass(:,ivm,ileaf,icarbon) .GT. zero) .AND. &
                (leaf_meanage(:,ivm) .GT. min_leaf_age_for_senescence(ivm)) .AND. &
                (t2m_month(:) .LT. ZeroCelsius) .AND. (t2m_week(:) .LT. t2m_month(:)) ) )
               
               ! Shed leaves, roots and fruits at a high rate = senescence
               turnover_time(:,ivm,ileaf) = leaffall(ivm)
               turnover_time(:,ivm,iroot) = leaffall(ivm)
               turnover_time(:,ivm,ifruit) = leaffall(ivm)

               ! Slowly turnover the structural carbon this allows to store 
               ! reserves. This structural pool is essential for the functioning 
               ! of the labile and reserve pools
               turnover_time(:,ivm,isapabove) = tau_eff_sap(:,ivm)
               senescence(:,ivm) = .TRUE.
 
            ENDWHERE
            
         ENDIF

       !! 3.1.5 Evergreen species 
       !  Evergreen species do not experience climatic senescence
       CASE ( 'none' )
    
       !! 3.1.6 Other cases
       !  In case no climatic senescence type is recognized.
       CASE default

          WRITE(numout,*) '  turnover: don''t know how to treat this PFT.'
          WRITE(numout,*) '  number (::ivm) : ',ivm
          WRITE(numout,*) '  senescence type (::senescence_type(ivm)) : ',senescence_type(ivm)

          STOP

       END SELECT

       !---TEMP---
       IF (ivm .EQ. test_pft .AND. ld_pheno) THEN
          WRITE(numout,*) 'phenology, senescence, ',senescence(:,ivm)
       ENDIF
       !----------

       !! 3.2 Drop leaves and roots, plus stems and fruits for grasses
       IF ( is_tree(ivm) ) THEN

          !! 3.2.1 Trees in climatic senescence 
          !  Trees in climate senescence lose their fine roots at the same rate as they lose their leaves. 
          !  The rate of biomass loss of both fine roots and leaves is presribed through the equation:
          !  \latexonly
          !  \input{turnover_clim_senes_biomass_eqn4.tex}
          !  \endlatexonly
          !  \n
          !  with ::leaffall(ivm) a PFT-dependent time constant which is given in ::stomate_constants).

          ! Notice that we do not change the carbohydrate (icarbres) and labile (ilabile) pools here.  We
          ! assume that trees can anticipate senescence and move these pools
          ! to more permanent storage areas so that are not lost, unlike
          ! in the case of harvesting.
          WHERE ( senescence(:,ivm) )

             ! Turnover at the stand level (gC m-2)
             turnover(:,ivm,ileaf,icarbon) = biomass(:,ivm,ileaf,icarbon) * dt / leaffall(ivm)
             turnover(:,ivm,iroot,icarbon) = biomass(:,ivm,iroot,icarbon) * dt / leaffall(ivm)

             ! Update biomass at the stand level (gC m-2)
             biomass(:,ivm,ileaf,icarbon) = biomass(:,ivm,ileaf,icarbon) - turnover(:,ivm,ileaf,icarbon)
             biomass(:,ivm,iroot,icarbon) = biomass(:,ivm,iroot,icarbon) - turnover(:,ivm,iroot,icarbon)

          ENDWHERE

          ! Turnover at tree level (gC tree-1)
          DO icirc = 1,ncirc

             WHERE ( senescence(:,ivm) )

                circ_class_biomass(:,ivm,icirc,ileaf,icarbon) = circ_class_biomass(:,ivm,icirc,ileaf,icarbon) * &
                     (un - dt / leaffall(ivm))
                circ_class_biomass(:,ivm,icirc,iroot,icarbon) = circ_class_biomass(:,ivm,icirc,iroot,icarbon) * &
                     (un - dt / leaffall(ivm))

             ENDWHERE

          ENDDO

       ! Grasses and crops
       ELSE

          !! 3.2.2.1 crops with 'crop' phenological model
          IF (senescence_type(ivm) .EQ. 'crop') THEN

             DO ipts = 1,npts
             
                IF (senescence(ipts,ivm)) THEN
             
                   ! Crops are planted every year. So make sure to remove everything 
                   ! at the end of senescence which for crops is actually harvest. 
                   ! Next stomate_prescribe should prescribe a new crop. If sapwood 
                   ! is not removed, some is left on site and the model will try to 
                   ! grow a crop. However, there are not enough reserves left so the 
                   ! crop will die. This results in one year of normal crop growth 
                   ! (following the prescribe) and then one year of almost no growth 
                   ! (the year starting from the too low reserves). This issue has 
                   ! been fixed.
                   CALL crop_harvest(npts, ipts, ivm, dt, &
                        veget_max, turnover, harvest_pool, harvest_type, &
                        harvest_cut, harvest_area, resolution, biomass, &
                        ind, leaf_meanage)   

                ENDIF

             ENDDO

          !! 3.2.2.2 grass or crops based on 'mixed' phenological model
          ! Phenological turnover
          ELSEIF (senescence_type(ivm) .EQ. 'mixed') THEN

             WHERE (senescence(:,ivm))

                turnover(:,ivm,ileaf,icarbon) = biomass(:,ivm,ileaf,icarbon) * dt / turnover_time(:,ivm,ileaf)
                turnover(:,ivm,isapabove,icarbon) = biomass(:,ivm,isapabove,icarbon) * dt / turnover_time(:,ivm,isapabove) 
                turnover(:,ivm,iroot,icarbon) = biomass(:,ivm,iroot,icarbon) * dt / turnover_time(:,ivm,iroot) 
                turnover(:,ivm,ifruit,icarbon) = biomass(:,ivm,ifruit,icarbon) * dt / turnover_time(:,ivm,ifruit)

             ENDWHERE

             ! Update the biomass pools for grasses and crops
             biomass(:,ivm,ileaf,icarbon) = biomass(:,ivm,ileaf,icarbon) - turnover(:,ivm,ileaf,icarbon)
             biomass(:,ivm,isapabove,icarbon) = biomass(:,ivm,isapabove,icarbon) - turnover(:,ivm,isapabove,icarbon)
             biomass(:,ivm,iroot,icarbon) = biomass(:,ivm,iroot,icarbon) - turnover(:,ivm,iroot,icarbon)
             biomass(:,ivm,ifruit,icarbon) = biomass(:,ivm,ifruit,icarbon) - turnover(:,ivm,ifruit,icarbon)

          !! 3.2.2.3 Grasses modeled as an evergreen biome  
          ELSEIF  (senescence_type(ivm) .EQ. 'none') THEN

             turnover(:,ivm,:,icarbon) = zero

          !! 3.2.2.4 Conceptual problem
          ELSE

             WRITE(numout,*) 'ERROR: senenescence type not known'
             
          END IF

          ! Synchronize circ_class_biomass and biomass
          DO ipar = 1,nparts

             WHERE (ind(:,ivm) .GT. zero)

                circ_class_biomass(:,ivm,1,ipar,icarbon) = &
                     biomass(:,ivm,ipar,icarbon) / ind(:,ivm)

             ELSEWHERE

                circ_class_biomass(:,ivm,1,ipar,icarbon) = zero

             ENDWHERE

          ENDDO ! nparts

       ENDIF      ! tree/grass

       !---TEMP---
       IF (ivm .EQ. test_pft .AND. ld_trnov) THEN
          WRITE(numout,*) 'Check turnover'
          !   DO ipar = 1,nparts
          !      WRITE(numout,*) 'biomass vs circ_class_biomass (gC m-2), ', biomass(1,ivm,ipar,icarbon), &
          !           SUM(circ_class_biomass(1,ivm,:,ipar,icarbon)*circ_class_n(1,ivm,:))
          !   ENDDO
          WRITE(numout,*) 'TOTAL biomass vs circ_class_biomass, ', SUM(biomass(1,ivm,:,icarbon)), &
               SUM( SUM(circ_class_biomass(1,ivm,:,:,icarbon),2) * circ_class_n(1,ivm,:),1)
       ENDIF
       !----------

    ENDDO        ! loop over PFTs


 !! 4. Leaf fall

    !  At a certain age, leaves fall off, even if the climate would allow a 
    !  green plant all year round. Even if the meteorological conditions are 
    !  favorable for leaf maintenance, plants, and in particular, evergreen 
    !  trees, have to renew their leaves simply because the old leaves become 
    !  inefficient. Another reason for leaf fall may be waterstress. When 
    !  the plant experiences water stress some of the leaves will die.
    !  Wstress is calculated from the ratio between stressed and unstressed
    !  GPP. If the wstress is less than 1, this implies that we have to maintain
    !  a complete canopy (and therefore have the Ra cost) but that only part 
    !  of the canopy will contribute to GPP. Although this is exactely what 
    !  happens on a warm summer day, what we are trying to account for here is
    !  the impact of a lasting drought (days to weeks). Adaptation to the
    !  growing conditions are accounted for through KF (see allocation).
    !   
    !  Roots, fruits (and stems) follow leaves. The decay rate varies with 
    !  leaf  age. Note that plant is not declared senescent in this case 
    !  (wchich is important for allocation: if the plant loses leaves because 
    !  of their age, it can renew them). 
    !
    !  Notice that we do not change the reserve pools here (ilabile and 
    !  icarbres). We assume that the tree will move these pools out of old 
    !  leaves and therefore the carbon will not be lost.  Trees are smart.
    !
    !  The leaf turnover rate due to aging of leaves is calculated using 
    !  the following equation:
    !  \latexonly
    !  \input{turnover_age_senes_biomass_eqn5.tex}
    !  \endlatexonly
    !  \n
    DO ivm = 2,nvm ! Loop over # PFTs

       !! 4.1 Calculate critical leaf age
       !  save old leaf mass
       lm_old(:) = biomass(:,ivm,ileaf,icarbon)

       !! initialize leaf mass change in age class
       delta_lm(:,:) = zero

       !  Trees and crops
       !  leaves, roots, fruits roots and fruits follow leaves.
       IF ( is_tree(ivm) .OR. (.NOT. natural(ivm)) ) THEN

          ! Critical age: prescribed for trees
          leaf_age_crit(:,ivm) = tau_eff_leaf(:,ivm)

       !  Grasses
       !  leaves, roots, fruits, sap follow leaves.
       ELSE

          !  Critical leaf age depends on long-term temperature
          !  generally, lower turnover in cooler climates.
          leaf_age_crit(:,ivm) = MIN( tau_eff_leaf(:,ivm) * leaf_age_crit_coeff(1) , &
               MAX( tau_eff_leaf(:,ivm) * leaf_age_crit_coeff(2) , &
               tau_eff_leaf(:,ivm) - leaf_age_crit_coeff(3) * &
               ( tlong_ref(:)-ZeroCelsius - leaf_age_crit_tref ) ) )

       END IF
          
       !! 4.2 Turnover for different leaf age classes
       DO ilage = 1, nleafages

          turnover_rate(:) = zero

          WHERE ( leaf_age(:,ivm,ilage) .GT. leaf_age_crit(:,ivm) / deux)

             turnover_rate(:) =  MIN( 0.99_r_std, dt / ( leaf_age_crit(:,ivm) * &
                  ( leaf_age_crit(:,ivm) / leaf_age(:,ivm,ilage) )**quatre ) )

          ENDWHERE
  
          IF (is_tree(ivm)) THEN

             ! Stand level turnover (gC m-2)
             ! Leaves
             ! Water stress is increasing (note that ::wstress is thus 
             ! decreasing)so the LAI will be lowered if the decrease is 
             ! small, this will result in a cosmetic change in LAI. 
             ! If water stress is increasing over a the course of a week, 
             ! this decrease may become important. The correction is 
             ! cummulative, for example, ::biomass(ileaf) is decreased 
             ! by 10% in day one and the remaining fraction of 
             ! ::biomass(ileaf) can be decreased by another 10% the next 
             ! day, etc ... The choice to multiply with ::wstress_month
             ! is arbitrary, we could have well used ::wstress_week. By
             ! using the monthly value we hope a smoother decrease 
             ! compared to using ::wstress_week or ::wstress_day.
             
             !+++CHECK+++
             ! Switch off the drought feedback
             ! Do we have enough biomass to satisfy the turnover?
!!$             WHERE ( (wstress_month(:,ivm)**(1/tau_hum_month)) .LT. un ) 
!!$                
!!$                ! Account for water stress
!!$                turnover_rate(:) =  turnover_rate(:) / &
!!$                     (wstress_month(:,ivm)**(1/tau_hum_month))
!!$
!!$             ENDWHERE
             !+++++++++++

             ! Update biomass and turnover pools
             dturnover(:) = biomass(:,ivm,ileaf,icarbon) * leaf_frac(:,ivm,ilage) * turnover_rate(:)
             biomass(:,ivm,ileaf,icarbon) = biomass(:,ivm,ileaf,icarbon) - dturnover(:) 
             turnover(:,ivm,ileaf,icarbon) = turnover(:,ivm,ileaf,icarbon) + dturnover(:)
             
             ! save leaf mass change
             delta_lm(:,ilage) = - dturnover(:)

             !+++CHECK+++
             ! It is not clear why roots follow the leaves. A turnover rate for roots based on
             ! tau_eff_root can be easily calculated
             ! Roots
             dturnover(:) = biomass(:,ivm,iroot,icarbon) * leaf_frac(:,ivm,ilage) * turnover_rate(:)
             biomass(:,ivm,iroot,icarbon) = biomass(:,ivm,iroot,icarbon) - dturnover(:)
             turnover(:,ivm,iroot,icarbon) = turnover(:,ivm,iroot,icarbon) + dturnover(:)
             !+++++++++++
             
             ! Fruit
             dturnover(:) = biomass(:,ivm,ifruit,icarbon) * leaf_frac(:,ivm,ilage) * turnover_rate(:)
             biomass(:,ivm,ifruit,icarbon) = biomass(:,ivm,ifruit,icarbon) - dturnover(:)
             turnover(:,ivm,ifruit,icarbon) = turnover(:,ivm,ifruit,icarbon) + dturnover(:)

             ! Tree level turnover (gC tree-1): leaves, roots and fruit
             DO icirc = 1,ncirc

                circ_class_biomass(:,ivm,icirc,ileaf,icarbon) = circ_class_biomass(:,ivm,icirc,ileaf,icarbon) * &
                     (un - leaf_frac(:,ivm,ilage) * turnover_rate(:))
                circ_class_biomass(:,ivm,icirc,iroot,icarbon) = circ_class_biomass(:,ivm,icirc,iroot,icarbon) * &
                     (un - leaf_frac(:,ivm,ilage) * turnover_rate(:))
                circ_class_biomass(:,ivm,icirc,ifruit,icarbon) = circ_class_biomass(:,ivm,icirc,ifruit,icarbon) * &
                     (un - leaf_frac(:,ivm,ilage) * turnover_rate(:))

             ENDDO

          ! Crops
          ELSEIF (.NOT. natural(ivm)) THEN

             ! The sapwood acts as the structure to supports leaves and roots and store the 
             ! reserves and labile. Because the growing season is short for crops, the 
             ! turnover of tissues should be kept to a minimum or the GPP and NPP will be 
             ! too low.
             dturnover(:) = biomass(:,ivm,ileaf,icarbon) * leaf_frac(:,ivm,ilage) * &
                  turnover_rate(:) / (wstress_month(:,ivm)**(1/tau_hum_month))

             ! Do we have enough biomass to satisfy the turnover?
             WHERE ( dturnover(:) .GE. biomass(:,ivm,ileaf,icarbon) )
                
                ! No longer account for wstress
                dturnover(:) = biomass(:,ivm,ileaf,icarbon) * &
                     leaf_frac(:,ivm,ilage) * turnover_rate(:)

             ENDWHERE

             ! Update biomass and turnover pools
             turnover(:,ivm,ileaf,icarbon) = turnover(:,ivm,ileaf,icarbon) + dturnover(:)
             biomass(:,ivm,ileaf,icarbon) = biomass(:,ivm,ileaf,icarbon) - dturnover(:)

             ! save leaf mass change
             delta_lm(:,ilage) = - dturnover(:)

             dturnover(:) = biomass(:,ivm,iroot,icarbon) * dt / tau_eff_root(:,ivm)
             turnover(:,ivm,iroot,icarbon) = turnover(:,ivm,iroot,icarbon) + dturnover(:)
             biomass(:,ivm,iroot,icarbon) = biomass(:,ivm,iroot,icarbon) - dturnover(:)

             dturnover(:) = biomass(:,ivm,ifruit,icarbon) * leaf_frac(:,ivm,ilage) * turnover_rate(:)
             turnover(:,ivm,ifruit,icarbon) = turnover(:,ivm,ifruit,icarbon) + dturnover(:)
             biomass(:,ivm,ifruit,icarbon) = biomass(:,ivm,ifruit,icarbon) - dturnover(:)
            
             ! Synchronize circ_class_biomass and biomass
             DO ipar = 1,nparts

                WHERE (ind(:,ivm) .GT. zero)

                   circ_class_biomass(:,ivm,1,ipar,icarbon) = &
                        biomass(:,ivm,ipar,icarbon) / ind(:,ivm)

                ELSEWHERE

                   circ_class_biomass(:,ivm,1,ipar,icarbon) = zero

                ENDWHERE

             ENDDO ! nparts

          ! Grasses   
          ELSEIF ( ( .NOT. is_tree(ivm) ) .AND. natural(ivm) ) THEN
             
             ! Grasses are now modeled as an evergreen biome, turnover is thus needed.
             ! This section was implemented much later then the previous blocks and we
             ! therefore followed our own advice and used tau_sap and tau_root rather 
             ! then having all biomass pool to follow leaf turnover.
             dturnover(:) = biomass(:,ivm,ileaf,icarbon) * leaf_frac(:,ivm,ilage) * &
                  turnover_rate(:) / (wstress_month(:,ivm)**(1/tau_hum_month))

             ! Do we have enough biomass to satisfy the turnover?
             WHERE ( dturnover(:) .GE. biomass(:,ivm,ileaf,icarbon) )
                
                ! No longer account for wstress
                dturnover(:) = biomass(:,ivm,ileaf,icarbon) * &
                     leaf_frac(:,ivm,ilage) * turnover_rate(:)

             ENDWHERE
             
             ! Grasses are no longer considered natural and so it is assumed that the
             ! aboveground turnover ends up in the harvest pool (mowing or grazing). 
             ! No idea whether this is anything close to reality but it is better to 
             ! assuming that there is no lateral transport. After testing over Europe
             ! the annual harvest_pool for grasses was 128 Tg C year compared to the 
             ! estimated/observed 161 Tg C year. It is left to grassland people to further
             ! improve this. Note that strictly speaking there is still no harvest, all 
             ! we did is moving the natural turnover (due to longevity) into the harvest
             ! pool.
             harvest_pool(:,ivm,1,icarbon) = harvest_pool(:,ivm,1,icarbon) + &
                  dturnover(:) * veget_max(:,ivm) * pixel_area(:)
             biomass(:,ivm,ileaf,icarbon) = biomass(:,ivm,ileaf,icarbon) - dturnover(:)

             ! save leaf mass change
             delta_lm(:,ilage) = - dturnover(:)

             ! Seeds are eaten
             dturnover(:) = biomass(:,ivm,ifruit,icarbon) * dt / tau_eff_leaf(:,ivm)
             harvest_pool(:,ivm,1,icarbon) = harvest_pool(:,ivm,1,icarbon) + &
                  dturnover(:) * veget_max(:,ivm) * pixel_area(:)
             biomass(:,ivm,ifruit,icarbon) = biomass(:,ivm,ifruit,icarbon) - dturnover(:)

             ! Half the sapwood is eaten, half is added to the litter
             dturnover(:) = biomass(:,ivm,isapabove,icarbon) * dt / tau_eff_sap(:,ivm)
             harvest_pool(:,ivm,1,icarbon) = harvest_pool(:,ivm,1,icarbon) + &
                  0.5 * dturnover(:) * veget_max(:,ivm) * pixel_area(:)
             turnover(:,ivm,isapabove,icarbon) = turnover(:,ivm,isapabove,icarbon) + &
                  0.5 * dturnover(:)
             biomass(:,ivm,isapabove,icarbon) = biomass(:,ivm,isapabove,icarbon) - dturnover(:)
 
             ! Fill associated harvest variables
             harvest_type(:,ivm) = ifm_grass
             harvest_cut(:,ivm) = icut_grass
             harvest_area(:,ivm) = veget_max(:,ivm) * pixel_area(:)

             ! Make sure the harvest does not enter the turnover pool
             turnover(:,ivm,ileaf,icarbon) = zero
             turnover(:,ivm,ifruit,icarbon) = zero

             ! Root turnover at the stand level (gC m-2). Roots are added to the litter pool
             dturnover(:) = biomass(:,ivm,iroot,icarbon) * dt / tau_eff_root(:,ivm)
             turnover(:,ivm,iroot,icarbon) = turnover(:,ivm,iroot,icarbon) + dturnover(:)
             biomass(:,ivm,iroot,icarbon) = biomass(:,ivm,iroot,icarbon) - dturnover(:)

             ! Synchronize circ_class_biomass and biomass
             DO ipar = 1,nparts

                WHERE (ind(:,ivm) .GT. zero)

                   circ_class_biomass(:,ivm,1,ipar,icarbon) = &
                        biomass(:,ivm,ipar,icarbon) / ind(:,ivm)

                ELSEWHERE

                   circ_class_biomass(:,ivm,1,ipar,icarbon) = zero

                ENDWHERE

             ENDDO ! nparts
             
          ELSE

             WRITE(numout,*) 'ERROR: turnover has not been defined'

          ENDIF ! is_tree(ivm)

          !---TEMP---
          IF (ivm .EQ. test_pft .AND. ld_trnov) THEN
             WRITE(numout,*) 'Check turnover - senesence'
             !   DO ipar = 1,nparts
             !      WRITE(numout,*) 'biomass vs circ_class_biomass (gC m-2), ', biomass(test_grid,ivm,ipar,icarbon), &
             !           SUM(circ_class_biomass(test_grid,ivm,:,ipar,icarbon)*circ_class_n(test_grid,ivm,:))
             !   ENDDO
             WRITE(numout,*) 'TOTAL biomass vs circ_class_biomass, ', SUM(biomass(test_grid,ivm,:,icarbon)), &
                  SUM( SUM(circ_class_biomass(test_grid,ivm,:,:,icarbon),2) * circ_class_n(test_grid,ivm,:),1)
          ENDIF
          !----------
             
       ENDDO

       !! 4.3 Recalculate the fraction of leaf biomass in each leaf age class.
       !  Older leaves will fall faster than younger leaves and therefore 
       !  the leaf age distribution needs to be recalculated after turnover. 
       !  The fraction of biomass in each leaf class is updated using the following equation:
       !  \latexonly
       !  \input{turnover_update_LeafAgeDistribution_eqn6.tex}
       !  \endlatexonly
       !  \n
       !  new fraction = new leaf mass of that fraction / new total leaf mass
       !               = (old fraction*old total leaf mass ::lm_old(:) + biomass change of that fraction ::delta_lm(:,m)  ) /
       !                  new total leaf mass ::biomass(:,ivm,ileaf
       DO ilage = 1, nleafages
          
          WHERE ( biomass(:,ivm,ileaf,icarbon) .GT. zero )

             leaf_frac(:,ivm,ilage) = ( leaf_frac(:,ivm,ilage)*lm_old(:) + delta_lm(:,ilage) ) / biomass(:,ivm,ileaf,icarbon)

          ELSEWHERE

             leaf_frac(:,ivm,ilage) = zero

          ENDWHERE
        
       ENDDO
       
    ENDDO         ! loop over PFTs


 !! 5. Definitely drop all leaves if there is a very low leaf mass during senescence.

    !  Both for deciduous trees and grasses same conditions are checked:
    !  If biomass is large enough (i.e. when it is greater than zero), 
    !  AND when senescence is set to true
    !  AND the leaf biomass drops below a critical minimum biomass level (i.e. when it is lower than half
    !  the minimum initial LAI ::lai_initmin(ivm) divided by the specific leaf area ::sla(ivm),
    !  ::lai_initmin(ivm) is set to 0.3 in stomate_data.f90 and sla is a constant that is set to 0.015366 m2/gC), 
    !  If these conditions are met, the flag ::shed_rest(:) is set to TRUE.
    !
    !  After this, the biomass of different carbon pools both for trees and grasses is set to zero
    !  and the mean leaf age is reset to zero.
    !  Finally, the leaf fraction and leaf age of the different leaf age classes is set to zero.
    DO ivm = 2,nvm ! Loop over # PFTs

       shed_rest(:) = .FALSE.

       !! 5.1 For deciduous trees: next to leaves, also fruits and fine roots are dropped 
       !  For deciduous trees: next to leaves, also fruits and fine roots are dropped: fruit ::biomass(:,ivm,ifruit) 
       !  and fine root ::biomass(:,ivm,iroot) carbon pools are set to zero.
       IF ( is_tree(ivm) .AND. ( senescence_type(ivm) .NE. 'none' ) ) THEN

          ! Check whether we shed the remaining leaves. The condition depends on biomass(ileaf)
          ! so first calculate the sheding at the tree level. because biomass(ileaf) is not 
          ! changed at the tree level the same statement can then be used at the stand level.
          DO icirc = 1,ncirc

             ! check whether we shed the remaining leaves
             WHERE ( ( biomass(:,ivm,ileaf,icarbon) .GT. zero ) .AND. senescence(:,ivm) .AND. &
                  ( biomass(:,ivm,ileaf,icarbon) .LT. (lai_initmin(ivm) / 2.)/sla(ivm) ) )

                ! Tree level (gC tree-1)
                circ_class_biomass(:,ivm,icirc,ileaf,icarbon)  = zero
                circ_class_biomass(:,ivm,icirc,iroot,icarbon)  = zero
                circ_class_biomass(:,ivm,icirc,ifruit,icarbon) = zero

             ENDWHERE

          ENDDO !ncirc

          ! check whether we shed the remaining leaves
          WHERE ( ( biomass(:,ivm,ileaf,icarbon) .GT. zero ) .AND. senescence(:,ivm) .AND. &
               ( biomass(:,ivm,ileaf,icarbon) .LT. (lai_initmin(ivm) / 2.)/sla(ivm) ) )

             shed_rest(:) = .TRUE.

             ! Stand level (gc m-2)
             turnover(:,ivm,ileaf,icarbon)  = turnover(:,ivm,ileaf,icarbon) + biomass(:,ivm,ileaf,icarbon)
             turnover(:,ivm,iroot,icarbon)  = turnover(:,ivm,iroot,icarbon) + biomass(:,ivm,iroot,icarbon)
             turnover(:,ivm,ifruit,icarbon) = turnover(:,ivm,ifruit,icarbon) + biomass(:,ivm,ifruit,icarbon)
             biomass(:,ivm,ileaf,icarbon)  = zero
             biomass(:,ivm,iroot,icarbon)  = zero
             biomass(:,ivm,ifruit,icarbon) = zero

             ! reset leaf age
             leaf_meanage(:,ivm) = zero

          ENDWHERE

       ENDIF

       !! 5.2 For grasses: all aboveground carbon pools, except the carbohydrate reserves are affected: 
       !  For grasses: all aboveground carbon pools, except the carbohydrate reserves are affected: 
       !  fruit ::biomass(:,ivm,ifruit,icarbon), fine root ::biomass(:,ivm,iroot,icarbon) and sapwood above 
       !  ::biomass(:,ivm,isapabove,icarbon) carbon pools are set to zero. 
       IF ( .NOT. is_tree(ivm) ) THEN

          IF (senescence_type(ivm) .EQ. 'crop') THEN

!!$             ! Shed the remaining leaves if LAI very low.
!!$             WHERE ( ( biomass(:,ivm,ileaf,icarbon) .GT. zero ) .AND. senescence(:,ivm) .AND. &
!!$                  (  biomass(:,ivm,ileaf,icarbon) .LT. (lai_initmin(ivm) / deux) / sla(ivm) ) )
!!$                
!!$                shed_rest(:) = .TRUE.
!!$             
!!$                ! Crops are planted every year. So make sure to remove everything at the end of senescence 
!!$                ! which for crops is actually harvest. Next stomate_prescribe should prescribe a new crop.
!!$                ! If sapwood is not removed, some is left on site and the model will try to grow a crop.
!!$                ! However, there are not enough reserves left so the crop will die. This results in one year
!!$                ! of normal crop growth (following the prescribe) and then one year of almost no growth (the
!!$                ! year starting from the too low reserves).
!!$                turnover(:,ivm,ileaf,icarbon) = turnover(:,ivm,ileaf,icarbon) + biomass(:,ivm,ileaf,icarbon) + &
!!$                     biomass(:,ivm,icarbres,icarbon) + biomass(:,ivm,ilabile,icarbon)
!!$                turnover(:,ivm,iroot,icarbon) = turnover(:,ivm,iroot,icarbon) + biomass(:,ivm,iroot,icarbon)
!!$                turnover(:,ivm,ifruit,icarbon) = turnover(:,ivm,ifruit,icarbon) + biomass(:,ivm,ifruit,icarbon)
!!$                turnover(:,ivm,isapabove,icarbon) = turnover(:,ivm,isapabove,icarbon) + biomass(:,ivm,isapabove,icarbon) + &
!!$                     biomass(:,ivm,isapbelow,icarbon)
!!$                turnover(:,ivm,isapabove,icarbon) = turnover(:,ivm,isapabove,icarbon) + biomass(:,ivm,iheartabove,icarbon) + &
!!$                     biomass(:,ivm,iheartbelow,icarbon)
!!$                biomass(:,ivm,isapabove,icarbon) = zero
!!$                biomass(:,ivm,ileaf,icarbon) = zero
!!$                biomass(:,ivm,iroot,icarbon) = zero
!!$                biomass(:,ivm,ifruit,icarbon) = zero
!!$                biomass(:,ivm,icarbres,icarbon) = zero
!!$                biomass(:,ivm,ilabile,icarbon) = zero
!!$                
!!$                ! To be sure put overlooked residuals also in the turnover pool 
!!$                biomass(:,ivm,iheartbelow,icarbon) = zero
!!$                biomass(:,ivm,iheartabove,icarbon) = zero
!!$                biomass(:,ivm,isapbelow,icarbon) = zero
!!$
!!$                ! No individuals left
!!$                ind(:,ivm) = zero
!!$
!!$                ! reset leaf age
!!$                leaf_meanage(:,ivm) = zero
!!$
!!$             ENDWHERE

          ELSEIF (senescence_type(ivm) .EQ. 'mixed') THEN

             ! Shed the remaining leaves if LAI very low.
             WHERE ( ( biomass(:,ivm,ileaf,icarbon) .GT. zero ) .AND. senescence(:,ivm) .AND. &
                  (  biomass(:,ivm,ileaf,icarbon) .LT. (lai_initmin(ivm) / deux) / sla(ivm) ) )
                
                shed_rest(:) = .TRUE.
             
                ! Grasses should live on after senescence. Do not shed the sapwood because then the whole 
                ! system collapse because the reserves are calculated based on the sapwood. no sapwood = no 
                ! reserves = no phenology = no growth.
                turnover(:,ivm,ileaf,icarbon) = turnover(:,ivm,ileaf,icarbon) + biomass(:,ivm,ileaf,icarbon) + &
                      biomass(:,ivm,icarbres,icarbon) + biomass(:,ivm,ilabile,icarbon)
                turnover(:,ivm,iroot,icarbon) = turnover(:,ivm,iroot,icarbon) + biomass(:,ivm,iroot,icarbon)
                turnover(:,ivm,ifruit,icarbon) = turnover(:,ivm,ifruit,icarbon) + biomass(:,ivm,ifruit,icarbon)
                biomass(:,ivm,ileaf,icarbon) = zero
                biomass(:,ivm,iroot,icarbon) = zero
                biomass(:,ivm,ifruit,icarbon) = zero
                biomass(:,ivm,icarbres,icarbon) = zero
                biomass(:,ivm,ilabile,icarbon) = zero

                ! Note that grasses stay alive so do not change the number of individuals!

                ! reset leaf age
                leaf_meanage(:,ivm) = zero
            
             ENDWHERE

          ELSEIF (senescence_type(ivm) .EQ. 'none') THEN

             ! Nothing should be done

          ELSE
             
             WRITE(numout,*) 'ERROR: senescence type is not known'

          ENDIF

          !---TEMP---
          IF (ivm .EQ. test_pft .AND. ld_pheno) THEN
             WRITE(numout,*) 'phenology, shed_rest, ',shed_rest(:)
          ENDIF
          !----------

          ! Synchronize circ_class_biomass and biomass
          DO ipar = 1,nparts

             WHERE (ind(:,ivm) .GT. zero)

                circ_class_biomass(:,ivm,1,ipar,icarbon) = &
                     biomass(:,ivm,ipar,icarbon) / ind(:,ivm)

             ELSEWHERE

                circ_class_biomass(:,ivm,1,ipar,icarbon) = zero

             ENDWHERE

          ENDDO ! nparts

       ENDIF ! is_tree(ivm)

       !---TEMP---
       IF (ivm .EQ. test_pft .AND. ld_trnov) THEN
          WRITE(numout,*) 'Check turnover - leaf sheding'
          WRITE(numout,*) 'ifruit biomass vs circ_class_biomass (gC m-2), ', biomass(test_grid,ivm,ifruit,icarbon), &
               SUM(circ_class_biomass(test_grid,ivm,:,ifruit,icarbon)*circ_class_n(test_grid,ivm,:))

          WRITE(numout,*) 'TOTAL biomass vs circ_class_biomass, ', SUM(biomass(test_grid,ivm,:,icarbon)), &
               SUM( SUM(circ_class_biomass(test_grid,ivm,:,:,icarbon),2) * circ_class_n(test_grid,ivm,:),1)
       ENDIF
       !----------

       !! 5.3 Reset the leaf age structure: the leaf fraction and leaf age 
       !  of the different leaf age classes is set to zero.
       DO ilage = 1, nleafages

          WHERE ( shed_rest(:) )

             leaf_age(:,ivm,ilage) = zero
             leaf_frac(:,ivm,ilage) = zero

          ENDWHERE

       ENDDO

    ENDDO          ! loop over PFTs
  

 !! 6. Herbivore activity: elephants, cows, gazelles but no lions.
 
    ! Herbivore activity affects the biomass of leaves and fruits as well 
    ! as stalks (only for grasses). Herbivore activity does not modify leaf 
    ! age structure. Herbivores ::herbivores(:,ivm) is the time constant of 
    ! probability of a leaf to be eaten by a herbivore, and is calculated in 
    ! ::stomate_season. following Mc Naughton et al. [1989].
    IF ( control%ok_herbivory ) THEN

       ! If the herbivore activity is allowed (if ::control%ok_herbivory is true, which is set in run.def), 
       ! remove the amount of biomass consumed by herbivory from the leaf biomass ::biomass(:,ivm,ileaf,icarbon) and 
       ! the fruit biomass ::biomass(:,ivm,ifruit,icarbon).
       ! The daily amount consumed equals the biomass multiplied by 1 day divided by the time constant ::herbivores(:,ivm).
       DO ivm = 2,nvm ! Loop over # PFTs

          IF ( is_tree(ivm) ) THEN

             !---TEMP---
             IF (ivm .EQ. test_pft .AND. ld_trnov) THEN
                WRITE(numout,*) 'herbivores, ',herbivores
             ENDIF
             !---------

             !! For trees: only the leaves and fruit carbon pools are affected
             !+++CHECK+++
             ! Why is root herbivory not accounted for. Could be difficult to parameterize but if 
             ! leaf browsing is accounted for, there is no reason to ignore root browsing
             WHERE (biomass(:,ivm,ileaf,icarbon) .GT. zero)

                WHERE (herbivores(:,ivm).GT. zero)

                   ! Stand level (gC m-2)
                   dturnover(:) = biomass(:,ivm,ileaf,icarbon) * dt / herbivores(:,ivm)
                   biomass(:,ivm,ileaf,icarbon) = biomass(:,ivm,ileaf,icarbon) - dturnover(:)
                   turnover(:,ivm,ileaf,icarbon) = turnover(:,ivm,ileaf,icarbon) + dturnover(:)
                   
                   dturnover(:) = biomass(:,ivm,ifruit,icarbon) * dt / herbivores(:,ivm)
                   biomass(:,ivm,ifruit,icarbon) = biomass(:,ivm,ifruit,icarbon) - dturnover(:)
                   turnover(:,ivm,ifruit,icarbon) = turnover(:,ivm,ifruit,icarbon) + dturnover(:)

                ENDWHERE

             ENDWHERE

             ! Tree level herbivory
             DO icirc = 1,ncirc

                !! For trees: only the leaves and fruit carbon pools are affected
                WHERE (biomass(:,ivm,ileaf,icarbon) .GT. zero)

                   WHERE (herbivores(:,ivm).GT. zero)
                      ! Tree level (gC tree-1)
                      circ_class_biomass(:,ivm,icirc,ileaf,icarbon) = circ_class_biomass(:,ivm,icirc,ileaf,icarbon) * &
                           (un - dt / herbivores(:,ivm))
                      circ_class_biomass(:,ivm,icirc,ifruit,icarbon) = circ_class_biomass(:,ivm,icirc,ifruit,icarbon) * & 
                           (un - dt / herbivores(:,ivm))
                   ENDWHERE

                ENDWHERE

             ENDDO ! ncirc
             !+++++++++

          ! grasses and croplands
          ELSE

             ! For grasses: all aboveground carbon pools are affected: leaves, fruits and sapwood above
             WHERE ( biomass(:,ivm,ileaf,icarbon) .GT. zero )

                WHERE (herbivores(:,ivm) .GT. zero)

                   dturnover(:) = biomass(:,ivm,ileaf,icarbon) * dt / herbivores(:,ivm)
                   turnover(:,ivm,ileaf,icarbon) = turnover(:,ivm,ileaf,icarbon) + dturnover(:)
                   biomass(:,ivm,ileaf,icarbon) = biomass(:,ivm,ileaf,icarbon) - dturnover(:)

                   dturnover(:) = biomass(:,ivm,isapabove,icarbon) * dt / herbivores(:,ivm)
                   turnover(:,ivm,isapabove,icarbon) = turnover(:,ivm,isapabove,icarbon) + dturnover(:)
                   biomass(:,ivm,isapabove,icarbon) = biomass(:,ivm,isapabove,icarbon) - dturnover(:)

                   dturnover(:) = biomass(:,ivm,ifruit,icarbon) * dt / herbivores(:,ivm)
                   turnover(:,ivm,ifruit,icarbon) = turnover(:,ivm,ifruit,icarbon) + dturnover(:)
                   biomass(:,ivm,ifruit,icarbon) = biomass(:,ivm,ifruit,icarbon) - dturnover(:)

                ENDWHERE

             ENDWHERE

             ! Synchronize circ_class_biomass and biomass
             DO ipar = 1,nparts

                WHERE (ind(:,ivm) .GT. zero)

                   circ_class_biomass(:,ivm,1,ipar,icarbon) = &
                        biomass(:,ivm,ipar,icarbon) / ind(:,ivm)

                ELSEWHERE

                   circ_class_biomass(:,ivm,1,ipar,icarbon) = zero

                ENDWHERE

             ENDDO ! nparts
             
          ENDIF  ! tree/grass?

          !---TEMP---
          IF (ivm .EQ. test_pft .AND. ld_trnov) THEN
             WRITE(numout,*) 'Check turnover - herbivory '
             WRITE(numout,*) 'TOTAL biomass vs circ_class_biomass, ', (biomass(test_grid,ivm,:,icarbon)), &
                  ( SUM(circ_class_biomass(test_grid,ivm,:,:,icarbon),2) * circ_class_n(test_grid,ivm,:))
          ENDIF
          !----------

       ENDDO    ! loop over PFTs

    ENDIF ! end herbivores


 !! 7. Fruit turnover for trees

    !  Fruit turnover for trees: trees simply lose their fruits
    !  with a time constant ::tau_fruit(ivm), 
    !  that is set to 90 days for all PFTs in ::constants_mtc
    !  Again, we assume that the trees do not lose either of the 
    !  reserve pools (ilabile or icarbres).
    DO iele = 1,nelements

       DO ivm = 2,nvm ! Loop over # PFTs

          IF ( is_tree(ivm) ) THEN

             ! Stand level (gC m-2)
             dturnover(:) = biomass(:,ivm,ifruit,iele) * dt / tau_fruit(ivm)
             biomass(:,ivm,ifruit,iele) = biomass(:,ivm,ifruit,iele) - dturnover(:)
             turnover(:,ivm,ifruit,iele) = turnover(:,ivm,ifruit,iele) + dturnover(:)

             ! Tree level (gC tree-1)
             circ_class_biomass(:,ivm,:,ifruit,iele) = circ_class_biomass(:,ivm,:,ifruit,iele) * &
                (un - dt / tau_fruit(ivm))

             !---TEMP---
             IF (ivm .EQ. test_pft .AND. ld_trnov) THEN
                WRITE(numout,*) 'Check turnover - fruits'
                WRITE(numout,*) 'biomass(ifruit), circ_class_biomass(ifruit), ',biomass(test_grid,ivm,ifruit,icarbon),  &
                     SUM(circ_class_biomass(test_grid,ivm,:,ifruit,icarbon)*circ_class_n(test_grid,ivm,:))
                WRITE(numout,*) 'turnover, ', dt/tau_fruit(ivm)
             ENDIF
             !----------

          ENDIF ! is_tree 

          !---TEMP---
          IF (ivm .EQ. test_pft .AND. ld_trnov) THEN
             WRITE(numout,*) 'biomass vs circ_class_biomass (gC m-2), ', biomass(test_grid,ivm,ifruit,icarbon), &
                  SUM(circ_class_biomass(test_grid,ivm,:,ifruit,icarbon)*circ_class_n(test_grid,ivm,:))
             WRITE(numout,*) 'TOTAL biomass vs circ_class_biomass, ', SUM(biomass(test_grid,ivm,:,icarbon)), &
                  SUM( SUM(circ_class_biomass(test_grid,ivm,:,:,icarbon),2) * circ_class_n(test_grid,ivm,:),1)
          ENDIF
          !----------

       ENDDO ! loop over PFTs

    END DO ! nelements


 !! 8. Conversion of sapwood to heartwood both for aboveground and belowground sapwood and heartwood.

    !  Following LPJ (Sitch et al., 2003), sapwood biomass is converted into heartwood biomass 
    !  with a time constant tau ::tau_sap(j) of 1 year.
    !  Note that this biomass conversion is not added to "turnover" as the biomass is not lost!
    DO ivm = 2,nvm ! Loop over # PFTs
   
       IF ( is_tree(ivm) ) THEN

          ! For the recalculation of age in 9.2 (in case the vegetation is not dynamic ie. ::control%ok_dgvm is FALSE), 
          ! the heartwood above and below is stored in ::hw_old(:).
          IF ( .NOT. control%ok_dgvm ) THEN

             hw_old(:) = biomass(:,ivm,iheartabove,icarbon) + biomass(:,ivm,iheartbelow,icarbon)
             sw_old(:) = biomass(:,ivm,isapabove,icarbon) + biomass(:,ivm,isapbelow,icarbon)

          ENDIF

          !! 8.1 Calculate the rate of sapwood to heartwood conversion 
          !  Calculate the rate of sapwood to heartwood conversion with the 
          !  time constant ::tau_sap(ivm) and update aboveground and 
          !  belowground sapwood ::biomass(:,ivm,isapabove) and ::biomass(:,ivm,isapbelow)
          !  and heartwood ::biomass(:,ivm,iheartabove) and ::biomass(:,ivm,iheartbelow).
          DO iele = 1,nelements
             
             ! Stand level above the ground (gC m-2)
             sapconv(:) = biomass(:,ivm,isapabove,iele) * dt / tau_eff_sap(:,ivm)
             biomass(:,ivm,iheartabove,iele) =  biomass(:,ivm,iheartabove,iele) + sapconv(:)
             biomass(:,ivm,isapabove,iele) = biomass(:,ivm,isapabove,iele) - sapconv(:)
                
             ! Stand level below the ground (gC m-2)
             sapconv(:) = biomass(:,ivm,isapbelow,iele) * dt / tau_eff_sap(:,ivm)
             biomass(:,ivm,iheartbelow,iele) =  biomass(:,ivm,iheartbelow,iele) + sapconv(:)
             biomass(:,ivm,isapbelow,iele) = biomass(:,ivm,isapbelow,iele) - sapconv(:)
             
             DO icirc = 1,ncirc
                ! Tree level above and belowground (gC tree-1)
                circ_class_biomass(:,ivm,icirc,iheartabove,iele) =  &
                     circ_class_biomass(:,ivm,icirc,iheartabove,iele) + &
                     circ_class_biomass(:,ivm,icirc,isapabove,iele) * dt / &
                     tau_eff_sap(:,ivm)
                circ_class_biomass(:,ivm,icirc,isapabove,iele) = &
                     circ_class_biomass(:,ivm,icirc,isapabove,iele) * &
                     (un - dt / tau_eff_sap(:,ivm)) 
                circ_class_biomass(:,ivm,icirc,iheartbelow,iele) = &
                     circ_class_biomass(:,ivm,icirc,iheartbelow,iele) + &
                     circ_class_biomass(:,ivm,icirc,isapbelow,iele) * dt / &
                     tau_eff_sap(:,ivm)
                circ_class_biomass(:,ivm,icirc,isapbelow,iele) = &
                     circ_class_biomass(:,ivm,icirc,isapbelow,iele) * &
                     (un - dt / tau_eff_sap(:,ivm))  
                
             ENDDO

          ENDDO

          !---TEMP---
          IF (ivm .EQ. test_pft .AND. ld_trnov) THEN
             WRITE(numout,*) 'Check turnover - sap & heartwood'
          !   DO ipar = 1, nparts
          !      WRITE(numout,*) 'biomass vs circ_class_biomass (gC m-2), ', biomass(test_grid,ivm,ipar,icarbon), &
          !           SUM(circ_class_biomass(test_grid,ivm,:,ipar,icarbon)*circ_class_n(test_grid,ivm,:))
          !   ENDDO
             WRITE(numout,*) 'TOTAL biomass vs circ_class_biomass, ', SUM(biomass(test_grid,ivm,:,icarbon)), &
                  SUM( SUM(circ_class_biomass(test_grid,ivm,:,:,icarbon),2) * circ_class_n(test_grid,ivm,:),1)
          ENDIF
          !----------

          !! 8.2 If the vegetation is not dynamic, the age of the plant is decreased. 
          !  The updated heartwood, the sum of new heartwood above and new heartwood below after 
          !  converting sapwood to heartwood, is saved as ::hw_new(:) .
          !  Creation of new heartwood decreases the age of the plant with a factor that is determined by: 
          !  old heartwood ::hw_old(:) divided by the new heartwood ::hw_new(:)
          IF ( .NOT. control%ok_dgvm ) THEN

             hw_new(:) = biomass(:,ivm,iheartabove,icarbon) + biomass(:,ivm,iheartbelow,icarbon)
             sw_new(:) = biomass(:,ivm,isapabove,icarbon) + biomass(:,ivm,isapbelow,icarbon)

             WHERE ( hw_new(:) .GT. zero )!

                age(:,ivm) = age(:,ivm) * hw_old(:)/hw_new(:)

                ! Sapwood age doesn't seem to be used, but it causes problems because it is not initialised in this
                ! routine and it is a local variable
                !!$ sapwood_age(:,ivm) = sapwood_age(:,ivm) * sw_old(:)/sw_new(:)

             ENDWHERE

          ENDIF

       ENDIF

    ENDDO       ! loop over PFTs

    !---TEMP---  
    IF (ld_trnov) THEN
       WRITE(numout,*) 'leaf_meanage in turnover', leaf_meanage(:,test_pft)
       WRITE(numout,*) 'leaf_age in turnover', leaf_age(:,test_pft,:)
       WRITE(numout,*) 'leaf_frac in turnover', leaf_frac(:,test_pft,:)
    ENDIF
    !----------

    ! Write mean leaf age and time constant of probability of a leaf to be eaten by a herbivore 
    ! to the stomate output file. 
    CALL histwrite_p (hist_id_stomate, 'LEAFAGE', itime, &
         leaf_meanage, npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'HERBIVORES', itime, &
         herbivores, npts*nvm, horipft_index)


 !! 10. Check mass balance closure

    !! 10.1 Calculate pools at the end of the routine
    pool_end = zero
    DO ipar = 1,nparts
       DO iele = 1,nelements
          pool_end(:,:,iele) = pool_end(:,:,iele) + &
               (biomass(:,:,ipar,iele) * veget_max(:,:)) + &
               (turnover(:,:,ipar,iele) * veget_max(:,:))
       ENDDO
    ENDDO

    ! The biomass harvest pool is expressed in gC pixel-1 So, it 
    ! shouldn't be multiplied by veget_max but it should be divided
    ! by pixel_area to obtain gC m-2.
    DO ivm = 1,nvm
       pool_end(:,ivm,icarbon) = pool_end(:,ivm,icarbon) + &
            SUM(harvest_pool(:,ivm,:,icarbon),2) / &
            pixel_area(:)
    ENDDO

    !! 10.2 Calculate components of the mass balance
    check_intern(:,:,iatm2land,icarbon) = zero
    check_intern(:,:,iland2atm,icarbon) = -un * zero
    check_intern(:,:,ilat2out,icarbon) = zero
    check_intern(:,:,ilat2in,icarbon) = -un * zero
    check_intern(:,:,ipoolchange,icarbon) = -un * (pool_end(:,:,icarbon) - pool_start(:,:,icarbon))
    closure_intern = zero
    DO imbc = 1,nmbcomp
       closure_intern(:,:,icarbon) = closure_intern(:,:,icarbon) + check_intern(:,:,imbc,icarbon)
    ENDDO

    ! Write outcome
    DO ipts = 1,npts
       DO ivm = 1,nvm
          IF (closure_intern(ipts,ivm,icarbon) .LT. min_stomate .AND. &
               closure_intern(ipts,ivm,icarbon) .GT. -min_stomate) THEN
             IF (ld_massbal) WRITE(numout,*) 'Mass balance closure: stomate_turnover.f90', ipts, ivm
          ELSE
             WRITE(numout,*) 'Error: mass balance is not closed in stomate_turnover.f90'
             WRITE(numout,*) '   ipts,ivm; ', ipts,ivm
             WRITE(numout,*) '   Difference is, ', closure_intern(ipts,ivm,icarbon)
             WRITE(numout,*) '   pool_end,pool_start: ', pool_end(ipts,ivm,icarbon), &
                  pool_start(ipts,ivm,icarbon)
         ENDIF
       ENDDO
    ENDDO

    IF (bavard.GE.4) WRITE(numout,*) 'Leaving prognostic turnover'

  END SUBROUTINE turnover_prognostic

END MODULE stomate_turnover
